from whacc import utils, image_tools
from whacc.feature_maker import feature_maker, total_rolling_operation_h5_wrapper
import matplotlib.pyplot as plt
from tqdm.auto import tqdm
import numpy as np
import h5py
import seaborn as sns
import os
import numpy as np
import cv2

# run quoted code below to make the original mini file
"""
## a) use this range [14537, 14607]
base_name = 'AH0407_160613_JC1003_AAAC_3lag.h5'
# /Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/Data/Jon/AH0407/160613/AH0407x160609-144.mp4
img_h5 = utils.lister_it(utils.get_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/DATA_FULL_in_range_only/'), base_name)
feature_h5 = utils.lister_it(utils.get_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/FEATURE_DATA/colab data/curation_for_auto_curator/DATA_FULL_in_range_only/data_AH0407_160613_JC1003_AAAC/3lag/'), base_name)
inds = [14419, 16417]
with h5py.File(img_h5[0], 'r') as h:
    images = h['images'][inds[0]:inds[1]]
    labels = h['labels'][inds[0]:inds[1]]
    frame_nums = np.asarray([len(labels)])
with h5py.File(feature_h5[0], 'r') as h:
    FD__original = h['FD__original'][inds[0]:inds[1]]

with h5py.File('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature.h5', 'w') as h:
    h['images'] = images
    h['labels'] = labels
    h['frame_nums'] = frame_nums
    h['FD__original'] = FD__original
    h['base_name'] = base_name.encode("ascii", "ignore")
    h['img_h5'] = img_h5[0].encode("ascii", "ignore")
    h['feature_h5'] = feature_h5[0].encode("ascii", "ignore")
"""
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################

h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature.h5'

FM = feature_maker(h5_in, operational_key='FD__original', delete_if_exists=True)

for periods in tqdm([-5, -4, -3, -2, -1, 1, 2, 3, 4, 5]):
    data, key_name = FM.shift(periods, save_it=True)

for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
    data, key_name = FM.rolling(smooth_it_by, 'mean', save_it=True)

for periods in tqdm([-50, -20, -10, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 10, 20, 50]):
    data, key_name = FM.operate('diff', kwargs={'periods': periods}, save_it=True)

for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
    data, key_name = FM.rolling(smooth_it_by, 'std', save_it=True)

win = 1
# key_to_operate_on = 'FD__original'
op = np.std
mod_key_name = 'FD_TOTAL_std_' + str(win) +  '_of_'
all_keys = utils.lister_it(utils.print_h5_keys(FM.h5_in, 1, 0), 'FD__', 'FD_TOTAL')
for key_to_operate_on in tqdm(all_keys):
    data_out = total_rolling_operation_h5_wrapper(FM, win, op, key_to_operate_on, mod_key_name = mod_key_name, save_it = True)

##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################

# utils.print_h5_keys(h5_in)
with h5py.File(h5_in, 'r') as h:
    images = h['images'][:]
    labels = h['labels'][:]
    frame_nums = h['frame_nums'][:]
    base_name = h['base_name']
    FD__original = h['FD__original'][:]




utils.open_folder('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM')

fps = 20
fourcc = cv2.VideoWriter_fourcc(*'XVID')
mp4_name = h5_in.replace('.h5', '.mp4')
width = 61
hieght = 61
channel = 3
video = cv2.VideoWriter(mp4_name, fourcc, float(fps), (width, hieght))
for img in images:
    img = np.stack(([img[:, :, 2], img[:, :, 1], img[:, :, 0]]), axis = 2) # resort for because order is reverse of numpy.imshow
    video.write(img)
video.release()


fps = 20
fourcc = cv2.VideoWriter_fourcc(*'XVID')
mp4_name = h5_in.replace('.h5', '_features.mp4')
width = 32
hieght = 64
channel = 1




FD_img = FD__original-np.min(FD__original)
FD_img = ((FD_img/np.max(FD_img))*255).astype(np.uint8)
cc = np.corrcoef(FD_img, rowvar=False)
cc, sorted_inds = cluster_corr(cc)
plt.figure()
sns.heatmap(cc)


FD_img = FD_img[:, sorted_inds]
# sorted_inds = np.argsort(np.mean(FD_img[labels==1, :], axis=0))
# FD_img = FD_img[:, sorted_inds]

# FD_img = np.vstack([FD_img[:, k] for k in sorted_inds]).T

for img in FD_img:
    # img = FD_img[k, :]
    img = img.reshape([hieght, width])
    img = np.repeat(img[:, :, None], 3, axis = 2)
    video.write(img)
video.release()
####################################
from pathlib import Path
from PIL import Image
expand_by = 1000
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature_pngs_div_by_3/'
Path(bd).mkdir(parents=True, exist_ok=True)
# bd = '/Users/phil/Desktop/example_feature_pngs/'
for i, (img, pole_img) in enumerate(tqdm(zip(FD_img, images), total = len(images))):
    img = img.reshape([hieght, width])
    img = cv2.resize(img, dsize=(expand_by, expand_by), interpolation=cv2.INTER_AREA)
    img = np.repeat(img[:, :, None], 3, axis = 2)

    # pole_img = np.stack(([pole_img[:, :, 2], pole_img[:, :, 1], pole_img[:, :, 0]]), axis = 2) # resort for because order is reverse of numpy.imshow
    pole_img = cv2.resize(pole_img, dsize=(expand_by, expand_by), interpolation=cv2.INTER_AREA)
    img = np.hstack((pole_img, img))
    im = Image.fromarray(img)
    im.save(bd+'frame'+"{:08d}".format(i)+'.png')

cd_to = """cd """ + '''"'''+bd+'''"'''
mp4_file_name = 'out10.mp4'
os.system(cd_to + """ && ffmpeg -framerate 20 -i frame%08d.png -c:v libx264 -crf 0 """ + mp4_file_name)

mp4_file_name = 'out10_5fps.mp4'
os.system(cd_to + """ && ffmpeg -framerate 5 -i frame%08d.png -c:v libx264 -crf 0 """ + mp4_file_name)

#below for quick time compatible
mp4_file_name = 'movie2.mp4'
os.system(cd_to + """ && ffmpeg -framerate 20 -i frame%08d.png -vcodec libx264 -pix_fmt yuv420p -lossless 1  """ + mp4_file_name)

# /Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/Data/Jon/AH0407/160613/AH0407x160609-144.mp4

####################################

from PIL import Image
expand_by = 1000
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature_pngs/'
# bd = '/Users/phil/Desktop/example_feature_pngs/'
for i, img in enumerate(tqdm(FD_img)):
    img = img.reshape([hieght, width])
    img = cv2.resize(img, dsize=(expand_by, expand_by), interpolation=cv2.INTER_AREA)
    img = np.repeat(img[:, :, None], 3, axis = 2)
    im = Image.fromarray(img)
    im.save(bd+'frame'+"{:08d}".format(i)+'.png')

cd_to = """cd """ + '''"'''+bd+'''"'''
mp4_file_name
os.system(cd_to + """ && ffmpeg -framerate 20 -i frame%08d.png -c:v libx264 -crf 0 """ + mp4_file_name)


cd "/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature_pngs3"

ffmpeg -framerate 20 -i frame%08d.png -vcodec libx264 -pix_fmt yuv420p -lossless 1 movie.mp4
# plt.figure()
# sns.heatmap(tmp1)
# plt.figure()
# sns.heatmap(np.tile(tmp1, 10))
#
# # cd "/Users/phil/Desktop/example_feature_pngs/"
# # "/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature_pngs/"

ffmpeg -framerate 20 -i frame%08d.png -c:v libx264 -crf 0 output3.mp4

ffmpeg -framerate 20 -i frame%08d.png -c:v copy output.mkv

ffmpeg -framerate 20 -i frame%08d.png -c:v libvpx-vp9 -pix_fmt yuva420p -lossless 1 out.webm

####################################
#
# from PIL import Image
# from subprocess import Popen, PIPE
#
# fps, duration = 24, 100
# p = Popen(['ffmpeg', '-y', '-f', 'image2pipe', '-vcodec', 'mjpeg', '-r', '24', '-i', '-', '-vcodec', 'mpeg4', '-qscale', '1', '-r', '24', 'video.avi'], stdin=PIPE)
# tmp1 = ['ffmpeg', '-y', '-f', 'image2pipe', '-vcodec', 'mjpeg', '-r', '24', '-i', '-', '-vcodec', 'mpeg4', '-qscale', '1', '-r', '24', 'video.avi']
# # p = Popen(['ffmpeg', '-y', '-f', 'image2pipe', '-c', 'v r10k', 'video.avi'], stdin=PIPE)
#
# for img in FD_img:
#     img = img.reshape([hieght, width])
#     img = np.repeat(img[:, :, None], 3, axis = 2)
#     im = Image.fromarray(img)
#     im.save(p.stdin, 'JPEG')
# p.stdin.close()
# p.wait()



with h5py.File('/Users/phil/Desktop/0006_cp.hdf5', 'r') as h:
    for k in h:
        try:
            print(type(h[k][:]))
        except:
            pass



from whacc import utils, image_tools
from whacc import feature_maker
import copy
import numpy as np
import pandas as pd
import h5py
from whacc import utils
from whacc.utils import info
import h5py
from whacc import model_maker, image_tools
import numpy as np

from datetime import datetime


from tqdm.autonotebook import tqdm


fn = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here_FINISHED/Jon/AH0407/160613/AH0407x160609_final_to_combine_1_to_5_of_5.h5'
from whacc import utils
utils.print_h5_keys(fn)
# def total_rolling_sliding_window_view(data_in, win, operation_function, shift_from_center = 0):
#   assert win%2 == 1, 'window must be odd'
#   mid = win//2
#   pad = np.zeros([win, data_in.shape[1]])*np.nan
#
#   L_pad = pad[:mid-shift_from_center]
#   R_pad = pad[:mid+shift_from_center]
#
#   data_in = np.vstack([L_pad, data_in, R_pad])
#   w = data_in.shape[1]
#   data_in = np.lib.stride_tricks.sliding_window_view(data_in, (win, w))
#   data_in = np.reshape(data_in, [-1, win*w])
#   data_out = operation_function(data_in, axis = 1)
#   return np.asarray(data_out)
#
# def rolling_std_with_frame_nums(data_frame_in, frame_nums, window):
#     out = []
#     # if add_name_str is None:
#     #     add_name_str = 'rolling_std_' + str(window)
#     # data_frame = data_frame_in.copy()
#     for i1, i2 in utils.loop_segments(frame_nums):
#         df_rolling = data_frame_in[i1:i2].rolling(window=window, min_periods=window)
#         x = df_rolling.std().shift(-(window // 2)).astype(np.float32)
#         out.append(x)
#     # data_frame = data_frame.add_suffix('_' + add_name_str)
#     return out
#
# frame_nums = [4000]*10
# data_in = np.random.random([4000*10, 2048])
# window = 5
#
#
#
# startTime = datetime.now()
# data_in_df = pd.DataFrame(data_in)
# test_2 = rolling_std_with_frame_nums(data_in_df, frame_nums, window)
# print(datetime.now() - startTime)
#
# startTime = datetime.now()
# test_1 = []
# for i1, i2 in utils.loop_segments(frame_nums):
#     x2 = np.lib.stride_tricks.sliding_window_view(data_in[i1:i2, :], [window, 4000])
#     x2 = np.std(x2, axis = 0)
#     test_1.append(np.squeeze(x2).T)
#
# # test_1 = []
# # for i1, i2 in utils.loop_segments(frame_nums):
# #     data_out_sliding_window_view = total_rolling_sliding_window_view(data_in[i1:i2, :], window, np.std, shift_from_center = 0)
# #     test_1.append(data_out_sliding_window_view)
# print(datetime.now() - startTime)
# test_1.shape
#
#
#
# test_1[0] == test_2[0]
#
# # x = np.random.random(10).reshape(5, 2)
# x = np.asarray([[0.46412716, 0.91757843],
#        [0.65178167, 0.58500641],
#        [0.37382417, 0.56820932],
#        [0.20973427, 0.7304579 ],
#        [0.66356314, 0.59616253]])
# x[:, 0] = 1
# print(np.std(x[:3, :], axis=0))
# print(np.std(x[1:4, :], axis=0))
# print(np.std(x[2:5, :], axis=0))
#
#
#
# np.reshape(x2, [-1, 3])
#
# test_1[0].shape, test_2[0].shape
#
#
#


h5_feature_data = '/Users/phil/Desktop/feature_example2_feature_data.h5'
data_in = image_tools.get_h5_key_and_concatenate(h5_feature_data, 'FD__original')

h5_feature_data = '/Users/phil/Desktop/small_test_3lag_feature_data.h5'
h5_new = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/small_h5s/3lag/small_test_3lag.h5'
in_gen = image_tools.ImageBatchGenerator(500, h5_new, label_key='labels')
RESNET_MODEL = model_maker.load_final_model()
utils.convert_h5_to_feature_h5(RESNET_MODEL, in_gen, h5_feature_data)

# h5_feature_data = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag_feature_data.h5'
# generate all the modified features (41*2048)+41 = 84,009
utils.standard_feature_generation(h5_feature_data)


h5_in = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/small_h5s/3lag/small_test_3lag.h5'



#
#
#
#
#
#
#
#
#
#
#
#
#
#

#
#
#
#
#
# window = 5
# min_periods = 5
#
# df_rolling = data_in.rolling(window=window, min_periods=min_periods, center=True)
# df_rolling.std
#
#
#
#
#
#
#
#
#


















class feature_maker2():
    def __init__(self, h5_in, frame_num_ind=None, frame_nums=None, operational_key='FD__original', disable_tqdm=False,
                 delete_if_exists=False, index_features_delete_the_rest=None):
        """

        Parameters
        ----------
        h5_in : h5 string pointing to the h4 file with the data to transform
        frame_num_ind : int referencing the frame num  ind to transform, note: if save_it is one ALL data is converted
        automatically and saved in h5. frame_num_ind only works when you call with
        frame_nums : default None, auto looks for key 'frame_nums' in h5 file or you can insert your own
        operational_key : the data array key to be transformed
        disable_tqdm : default False, when save_it is True it will show a loading bar with the progress unless set to true
        delete_if_exists : default False, when calling a function with save_it as True, you can choose to overwrite that
        data by setting this value to True

        Returns
        -------
        feature maker class

        Examples
        ________

        h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'

        FM = feature_maker(h5_in, frame_num_ind=2, delete_if_exists = True) #set frame_num_ind to an int so that we can quickly look at the transformation
        data, data_name = FM.shift(5, save_it=False) # shift it 5 to the right, fills in with nan as needed, look at just the 5th frame num set (5th video)
        # to see how it looks
        print('the below name will be used to save the data in the the h5 when calling functions with save_it=True')
        print(data_name)
        FM.shift(5, save_it=True) # now lets save it

        data, data_name_rolling_mean_100 = FM.rolling(100, 'mean', save_it=True)  #lets do the rolling mean and save it

        data, data_name = FM.operate('diff', kwargs={'periods': 1}, save_it=False) # lets perform a diff operation
        print(data_name)
        print(data)
        FM.operate('diff', kwargs={'periods': -1}, save_it=True) # now lets save it and save it

        # now lets change the operational key so we can transform some data we just transformed, specifically the rolling mean 10
        FM.set_operation_key(data_name_rolling_mean_100)

        data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=False) # lets check how the name will change
        print(data_name_diff_100_mean)
        print("notice the FD__ twice, this means the data has been transformed twice")
        print('also notice that how the data was transformed can be seen because it is split up by ____ (4 underscores)')

        data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=True) # save it

        a = utils.print_h5_keys(h5_in, 1, 1)
        key_name_list = ['FD__original', 'FD__FD__FD__original_rolling_mean_W_100_SFC_0_MP_100_____diff_periods_-50____', 'FD__FD__original_rolling_mean_W_100_SFC_0_MP_100____']
        with h5py.File(h5_in, 'r+') as h:
            for k in key_name_list:
                plt.plot(h[k][:8000, 0])
        """
        if utils.h5_key_exists(h5_in, 'has_data_been_randomized'):
            assert image_tools.get_h5_key_and_concatenate(h5_in,
                                                          'has_data_been_randomized').tolist() is False, """this data has been randomized, it is not fit to perform temporal operations on, search for key 'has_data_been_randomized' for more info"""
        self._frame_num_ind_save_ = None
        self.disable_tqdm = disable_tqdm
        self.h5_in = h5_in
        self.frame_num_ind = frame_num_ind
        self.delete_if_exists = delete_if_exists
        self.operational_key = operational_key
        if frame_nums is None:
            print('extracting frame_nums from h5 file, ideally you should just put that in yourself though')
            frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
        self.all_frame_nums = frame_nums
        self.len_frame_nums = len(self.all_frame_nums)
        self.set_data_and_frame(frame_num_ind)
        index_features_delete_the_rest

    def set_data_inds(self, ind):
        tmp_inds = np.asarray(utils.loop_segments(self.all_frame_nums, returnaslist=True))
        if ind is None:
            self.data_inds = [tmp_inds[0][0], tmp_inds[-1][-1]]
        else:
            self.data_inds = tmp_inds[:, ind]

    def set_operation_key(self, key_name=None):
        if key_name is not None:
            self.operational_key = key_name
        with h5py.File(self.h5_in, 'r') as h:
            self.full_data_shape = np.asarray(h[self.operational_key].shape)
            if self.frame_num_ind is None:
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][:]))
            else:
                a = self.data_inds
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][a[0]:a[1]]))

    def init_h5_data_key(self, data_key, num_f, delete_if_exists=False):
        key_exists = utils.h5_key_exists(self.h5_in, data_key)
        assert not (key_exists and not delete_if_exists), "key exists, if you want to overwrite set 'delete_if_exists' = True"
        with h5py.File(self.h5_in, 'r+') as x:
            if key_exists and delete_if_exists:
                print('deleting key to overwrite it')
                del x[data_key]
            L = len(x[self.operational_key][:, 0])
            init_data = np.ones((num_f, L))*np.nan
            # x.create_dataset_like(data_key, init_data)
            x.create_dataset(data_key, shape=[L, num_f])

    def rolling(self, window, operation, save_it=False, feature_subset_inds=None, shift_from_center=0, min_periods=None, kwargs={}):
        """

        Parameters
        ----------
        window : window size int
        operation : a string with the operation e.g. 'mean' or 'std' see pandas docs for rolling
        shift_from_center : default 0 but can shift as needed
        min_periods : default is equal to win length, only allows operation when we have this many data points. so deals with
        the edges
        save_it : bool, False, loop through frame nums and save to h5 file
        kwargs : dict of args that can be applied to 'operation' function

        Returns
        -------

        """
        if min_periods is None:
            min_periods = int(window)
        add_name_list = ['FD__' + self.operational_key + '_rolling_',
                         '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods) + '____']
        add_name_str = operation.join(add_name_list)
        if save_it:
            self._save_it_(self.rolling, window, operation, False, feature_subset_inds, shift_from_center, min_periods, kwargs)
            return None, add_name_str

        if feature_subset_inds is not None:
            data_frame = self.data.copy().iloc[:, feature_subset_inds]
        else:
            data_frame = self.data.copy()

        for i1, i2 in utils.loop_segments(self.frame_nums):
            df_rolling = data_frame[i1:i2].rolling(window=window, min_periods=min_periods, center=True)
            tmp_func = eval('df_rolling.' + operation)
            data_frame[i1:i2] = tmp_func(**kwargs).shift(shift_from_center).astype(np.float32)
        return data_frame, add_name_str

    def shift(self, shift_by, save_it=False, feature_subset_inds=None):
        """

        Parameters
        ----------
        shift_by : amount to shift by
        save_it :  bool, False, loop through frame nums and save to h5 file

        Returns
        -------

        """
        add_name_str = 'FD__' + self.operational_key + '_shift_' + str(shift_by) + '____'
        if save_it:
            self._save_it_(self.shift, shift_by, False, feature_subset_inds)
            return None, add_name_str

        if feature_subset_inds is not None:
            data_frame = self.data.copy().iloc[:, feature_subset_inds]
        else:
            data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = data_frame[i1:i2].shift(shift_by).astype(np.float32)
        return data_frame, add_name_str

    def operate(self, operation, save_it=False, feature_subset_inds=None, extra_name_str='', kwargs={}):
        """

        Parameters
        ----------
        operation : a string with the operation e.g. 'mean' or 'std' or 'diff' see pandas operation on  dataframes
        save_it :  bool, False, loop through frame nums and save to h5 file
        extra_name_str : add to key name string if desired
        kwargs : dict of args that can be applied to 'operation' function see pandas operation on  dataframes

        Returns
        -------

        """
        add_name_str_tmp = self.dict_to_string_name(kwargs)
        add_name_str = 'FD__' + self.operational_key + '_' + operation + '_' + add_name_str_tmp + extra_name_str + '____'
        if save_it:
            self._save_it_(self.operate, operation, False, feature_subset_inds, extra_name_str, kwargs)
            return None, add_name_str
        if feature_subset_inds is not None:
            data_frame = self.data.copy().iloc[:, feature_subset_inds]
        else:
            data_frame = self.data.copy()

        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = eval('data_frame[i1:i2].' + operation + '(**kwargs).astype(np.float32)')
        return data_frame, add_name_str

    @staticmethod
    def dict_to_string_name(in_dict):
        """
        used to transform dict into a string name for naming h5 keys
        Parameters
        ----------
        in_dict : dict

        Returns
        -------
        string
        """
        str_list = []
        for k in in_dict:
            str_list.append(k)
            str_list.append(str(in_dict[k]))
        return '_'.join(str_list)

    def set_data_and_frame(self, ind):
        self.frame_num_ind = ind
        if ind is None:
            self.frame_nums = copy.deepcopy(self.all_frame_nums)
        else:
            self.frame_nums = [self.all_frame_nums[ind]]
        self.set_data_inds(ind)
        self.set_operation_key()

    # tqdm(sorted(random_frame_inds[i]), disable=disable_TQDM, total=total_frames, initial=cnt1)
    def _save_it_(self, temp_funct, *args):
        self._frame_num_ind_save_ = copy.deepcopy(self.frame_num_ind)
        with h5py.File(self.h5_in, 'r+') as h:
            for k in tqdm(range(self.len_frame_nums), disable=self.disable_tqdm):
                self.set_data_and_frame(k)
                print(temp_funct)
                print(args)
                data, key_name = temp_funct(*args)
                if k == 0:
                    self.init_h5_data_key(key_name, data.shape[1], delete_if_exists=self.delete_if_exists)
                    print('making key, ' + key_name)
                a = self.data_inds
                h[key_name][a[0]:a[1]] = data
        self.set_data_and_frame(self._frame_num_ind_save_)  # set data back to what it was when user set it


def total_rolling_operation(data_in, win, operation_function, shift_from_center=0):
    """
    NOTE: for making feature data proper key names for saving is 'FD_TOTAL_' folowed by operation e.g. 'FD_TOTAL_nanstd'
    Parameters
    ----------
    data_in : 2D matrix
    win : window size
    operation_function : function to be applies to each window e.g. np.nanmean, note DON'T include parentheses
    shift_from_center : num units shift from center

    Returns
    -------
    data_out: output data
    is_nan_inds: bool array indexing where nans were
    """
    assert win % 2 == 1, 'window must be odd'
    mid = win // 2
    pad = np.zeros([win, data_in.shape[1]]) * np.nan

    L_pad = pad[:mid - shift_from_center]
    R_pad = pad[:mid + shift_from_center]

    data_in = np.vstack([L_pad, data_in, R_pad])

    is_nan_inds = []
    data_out = []
    for k in range(data_in.shape[0] - win + 1):
        x = data_in[k:(k + win)]
        data_out.append(operation_function(x))
        is_nan_inds.append(np.any(np.isnan(x)))
    return np.asarray(data_out), np.asarray(is_nan_inds)


def total_rolling_operation_h5_wrapper(FM, window, operation, key_to_operate_on, mod_key_name=None, save_it=False,
                                       shift_from_center=0):
    if save_it:
        assert mod_key_name is not None, """if save_it is True, 'mod_key_name' must not be None e.g. 'FD_TOTAL_std_1_of_'"""
    all_data = []
    with h5py.File(FM.h5_in, 'r') as h:
        frame_nums = h['frame_nums'][:]
        for i, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
            data_out, is_nan_inds = total_rolling_operation(h[key_to_operate_on][i1:i2, :], window, operation,
                                                            shift_from_center=shift_from_center)
            all_data.append(data_out)
    all_data = np.hstack(all_data)
    mod_key_name = mod_key_name + key_to_operate_on
    if save_it:
        utils.overwrite_h5_key(FM.h5_in, mod_key_name, all_data)
    return all_data


def convert_h5_to_feature_h5(model, in_generator, h5_new_full_file_name=None):
    assert len(in_generator.H5_file_list) == 1, 'generator must be made from a single H5 file for now can change later'
    # this is due to needing to copy over the other keys like frame nums will need to combine it!!
    # see below line
    # utils.copy_over_all_non_image_keys(in_generator.H5_file_list[0], h5_new_full_file_name)
    if h5_new_full_file_name is None:
        if len(in_generator.H5_file_list) == 1:
            h5_new_full_file_name = in_generator.H5_file_list[0].replace('.h5', '_feature_data.h5')
        else:
            assert False, """if generator contains more than one file, 'h5_new_full_file_name' can not be none, please name it yourself"""

    h5c = image_tools.h5_iterative_creator(h5_new_full_file_name,
                                           overwrite_if_file_exists=True,
                                           max_img_height=1,
                                           max_img_width=2048,
                                           close_and_open_on_each_iteration=True,
                                           color_channel=False,
                                           add_to_existing_H5=False,
                                           ignore_image_range_warning=False,
                                           dtype_img=h5py.h5t.IEEE_F32LE,
                                           dtype_labels=h5py.h5t.IEEE_F32LE)

    for k in tqdm(range(in_generator.__len__())):
        x, y = in_generator.__getitem__(k)
        features = model.predict(x)
        h5c.add_to_h5(features, y)
    with h5py.File(h5_new_full_file_name, 'r+') as h:
        h['FD__original'] = h['images'][:]
        del h['images']

    utils.copy_over_all_non_image_keys(in_generator.H5_file_list[0], h5_new_full_file_name)


def get_feature_data_names(feature_list, n=2048):

    featuredata_names = [k.replace('FD__', '').replace('____', '') for k in feature_list]
    final_feature_names = []
    names = []
    nums = []
    for i, (k1, k2) in enumerate(zip(featuredata_names, feature_list)):
        if '_TOTAL_' in k2:
            names.append(k1)
            nums.append(np.arange(1))
            final_feature_names.append(k1)
        else:

            names.append(np.repeat(k1, n))
            nums.append(np.arange(n))
            final_feature_names.append([i1 + '_' + str(i2) for i1, i2 in zip(names[-1], nums[-1])])
    out = [np.hstack(k) for k in [final_feature_names, names, nums, featuredata_names]]
    out[0] = list(out[0])
    out[-1] = list(out[-1])
    return out


def standard_feature_generation(h5_feature_data):
    FM = feature_maker(h5_feature_data, operational_key='FD__original', delete_if_exists=True)

    for periods in tqdm([-5, -4, -3, -2, -1, 1, 2, 3, 4, 5]):
        data, key_name = FM.shift(periods, save_it=True)

    for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
        data, key_name = FM.rolling(smooth_it_by, 'mean', save_it=True)

    for periods in tqdm([-50, -20, -10, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 10, 20, 50]):
        data, key_name = FM.operate('diff', kwargs={'periods': periods}, save_it=True)

    for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
        data, key_name = FM.rolling(smooth_it_by, 'std', save_it=True)

    win = 1
    # key_to_operate_on = 'FD__original'
    op = np.std
    mod_key_name = 'FD_TOTAL_std_' + str(win) + '_of_'
    all_keys = utils.lister_it(utils.print_h5_keys(FM.h5_in, 1, 0), 'FD__', 'FD_TOTAL')
    for key_to_operate_on in tqdm(all_keys):
        data_out = total_rolling_operation_h5_wrapper(FM, win, op, key_to_operate_on, mod_key_name=mod_key_name,
                                                      save_it=True)


def load_selected_features(h5_in, feature_index=None):
    """

    Parameters
    ----------
    h5_in : full directory of the full feature h5 file 84,009
    feature_index : bool array of len 84,009

    Returns
    -------

    """
    d = utils.load_feature_data()
    feature_list = d['feature_list_unaltered']
    if feature_index is None:
        feature_index= d['final_selected_features_bool']
        # feature_index = utils.get_selected_features(greater_than_or_equal_to=4)
    feature_index = utils.make_list(feature_index)

    if isinstance(h5_in, list):
        all_x = []
        all_y = []
        for k in h5_in:
            tmp_x, tmp_y = load_selected_features(k, feature_list)
            all_x.append(tmp_x)
            all_y.append(tmp_y)
            del tmp_x, tmp_y

        all_x = np.vstack(all_x)
        all_y = np.hstack(all_y)
        return all_x

    all_x = None
    for k in tqdm(feature_list):
        if all_x is None:
            all_x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
            inds = feature_index[:all_x.shape[1]]
            del feature_index[:all_x.shape[1]]
            all_x = all_x[:, inds]
        else:
            x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
            if len(x.shape) > 1:
                inds = feature_index[:x.shape[1]]
                del feature_index[:x.shape[1]]
                x = x[:, inds]
                all_x = np.hstack((all_x, x))
            else:
                inds = feature_index.pop(0)  # single true or false to include the "TOTAL" variables
                if inds:
                    all_x = np.hstack((all_x, x[:, None]))
    return all_x



import os
import shutil
import h5py
from whacc import feature_maker, utils
h5_in = '/Users/phil/Desktop/small_test_3lag_feature_data.h5'
h5_in_temp = h5_in.replace('.h5', '_TEMP.h5')
with h5py.File(h5_in, 'r+') as h:
    for k in h.keys():
        if k not in ['FD__original', 'frame_nums', 'labels', 'multiplier']:
            del h[k]
        else:
            print(k)
if os.path.isfile(h5_in_temp):
    os.remove(h5_in_temp)
shutil.copy(h5_in, h5_in_temp)

feature_maker.standard_feature_generation2(h5_in_temp)

utils.print_h5_keys(h5_in_temp)

video_directory = '/Users/phil/Desktop/pipeline_test/data/'
local_temp_dir = '/Users/phil/Desktop/pipeline_test/temp_dir/'
utils.make_mp4_list_dict(video_directory, overwrite=True)
utils.batch_process_videos_on_colab(video_directory, local_temp_dir, 2)


FM = feature_maker2(h5_in_temp, operational_key='FD__original', delete_if_exists=True)
for smooth_it_by in tqdm([3, 7, 11, 15, 41]):
    data, key_name = FM.rolling(smooth_it_by, 'mean', save_it=True)
for periods in tqdm([-3, -1, 2, 3]):
    data, key_name = FM.shift(periods, save_it=True)

all_TOTAL_keys = ['FD__FD__original_rolling_mean_W_11_SFC_0_MP_11____',
                   'FD__FD__original_rolling_mean_W_15_SFC_0_MP_15____',
                   'FD__FD__original_rolling_mean_W_3_SFC_0_MP_3____',
                   'FD__FD__original_rolling_mean_W_41_SFC_0_MP_41____',
                   'FD__FD__original_rolling_mean_W_7_SFC_0_MP_7____',
                   'FD__FD__original_shift_-1____', 'FD__FD__original_shift_-3____',
                   'FD__FD__original_shift_2____', 'FD__FD__original_shift_3____',
                   'FD__original']
win = 1
# key_to_operate_on = 'FD__original'
op = np.std
mod_key_name = 'FD_TOTAL_std_' + str(win) + '_of_'
for key_to_operate_on in tqdm(all_TOTAL_keys):
    data_out = total_rolling_operation_h5_wrapper(FM, win, op, key_to_operate_on, mod_key_name=mod_key_name,
                                                  save_it=True)

def index_out_pre_made_features(h5_in, feature_subset_inds, key):
    with h5py.File(h5_in, 'r+') as h:
        out = []
        feature_subset_inds.any()
        for k in np.where(feature_subset_inds)[0]:
            out.append(h[key][:, k])
        out = np.asarray(out).T
        del h[key]
        if feature_subset_inds.any():
            h[key] = out


d = utils.load_feature_data()
bool_2048_features_index = d['final_selected_features_bool'][41:]
feature_list_unaltered_regular = d['feature_list_unaltered'][41:]

# exclude set that are fully created already here when you have time to code it
cnt_ind = -1
for periods in tqdm([-50, -20, -10, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 10, 20, 50]):
    cnt_ind+=1
    feature_subset_inds = bool_2048_features_index[cnt_ind*2048:(1+cnt_ind)*2048]
    if feature_subset_inds.any():
        data, key_name = FM.operate('diff', feature_subset_inds=feature_subset_inds, kwargs={'periods': periods}, save_it=True)


for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
    cnt_ind+=1
    feature_subset_inds = bool_2048_features_index[cnt_ind*2048:(1+cnt_ind)*2048]
    if smooth_it_by in [3, 7, 11, 15, 41]:
        key = feature_list_unaltered_regular[cnt_ind]
        index_out_pre_made_features(FM.h5_in, feature_subset_inds, key)
    elif feature_subset_inds.any():
        data, key_name = FM.rolling(smooth_it_by, 'mean', feature_subset_inds=feature_subset_inds, save_it=True)


for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
    cnt_ind+=1
    feature_subset_inds = bool_2048_features_index[cnt_ind*2048:(1+cnt_ind)*2048]
    if feature_subset_inds.any():
        data, key_name = FM.rolling(smooth_it_by, 'std', feature_subset_inds=feature_subset_inds, save_it=True)


for periods in tqdm([-5, -4, -3, -2, -1, 1, 2, 3, 4, 5]):
    cnt_ind+=1
    feature_subset_inds = bool_2048_features_index[cnt_ind*2048:(1+cnt_ind)*2048]
    if periods in [-3, -1, 2, 3]:
        key = feature_list_unaltered_regular[cnt_ind]
        index_out_pre_made_features(FM.h5_in, feature_subset_inds, key)
    elif feature_subset_inds.any():
        data, key_name = FM.shift(periods, feature_subset_inds=feature_subset_inds, save_it=True)

cnt_ind+=1
feature_subset_inds = bool_2048_features_index[cnt_ind*2048:(1+cnt_ind)*2048]
index_out_pre_made_features(FM.h5_in, feature_subset_inds, 'FD__original')


# with h5py.File(FM.h5_in, 'r+') as h:
#     FD__original = []
#     for k in np.where(feature_subset_inds)[0]:
#         FD__original.append(h['FD__original'][:, k])
#     FD__original = np.asarray(FD__original).T
#     del h['FD__original']
#     # del h['FD__FD__original_shift_3____']
#     h['FD__original'] = FD__original

with h5py.File(FM.h5_in, 'r') as h:
    final_mat = []
    x = utils.lister_it(list(h.keys()), 'FD__')
    keys = natsorted(x, alg=ns.REAL)
    for k in keys:
        x = h[k][:]
        if len(x.shape)==1:
            x = x[:, None]
        final_mat.append(x)

final_mat = np.hstack(final_mat)


h5_in = '/Users/phil/Desktop/small_test_3lag_feature_data.h5'
h5_in_temp = h5_in.replace('.h5', '_TEMP.h5')
with h5py.File(h5_in, 'r+') as h:
    for k in h.keys():
        if k not in ['FD__original', 'frame_nums', 'labels', 'multiplier']:
            del h[k]
        else:
            print(k)
import shutil
shutil.copy(h5_in, h5_in_temp)
utils.print_h5_keys(h5_in_temp)


def get_example_h5_selected_2105_features(h5_feature_data=None):
    if h5_feature_data is None:
        h5_feature_data = '/Users/phil/Desktop/feature_example2_feature_data.h5'
    d = utils.load_feature_data()
    feature_index = d['final_selected_features_bool']

    with h5py.File(h5_feature_data, 'r') as h:
        all_x = []
        all_y = np.asarray(h['labels'][:])
        for k in d['feature_list_unaltered']:
            x = h[k][:]
            if len(x.shape) == 1:
                x = x[:, None]
            print(x.shape)
            all_x.append(x)
    all_x = np.hstack(all_x)
    print(all_x.shape)
    all_x = all_x[:, feature_index]
    return all_x, all_y

utils.standard_feature_generation(h5_in_temp)


all_x, all_y = get_example_h5_selected_2105_features(h5_in_temp)

import seaborn as sns

a, b = all_x, final_mat
tmp2 = ((a == b) | (np.isnan(a) & np.isnan(b)))
tmp2.all()
x = a-b
sns.heatmap(x[:, :20])
# sns.heatmap(all_x == final_mat)
sns.heatmap(a[:, :20])
plt.figure()
sns.heatmap(b[:, -20:])



a, b = all_x[:, :], final_mat[:, :-1]
tmp2 = ((a == b) | (np.isnan(a) & np.isnan(b)))
sns.heatmap(tmp2)

plt.plot(a[:, :5])

plt.figure()
plt.plot(b[:, :5])














tmp1 = load_selected_features(FM.h5_in, feature_index=None)

with h5py.File(FM.h5_in, 'r') as h:
    for k in h.keys():
        print(k)
        print(h[k][:].shape)


"""

FD__original
(250, 2048)
still need to index this but I want to keep these as well so make sure that works out

step 1 ) first check to make sure that when i load this it looks correct

step 2) now i get all the features and put them in one single final h5, so I want to do that here as well instead of saving it to the H5 
file as all the individual ones, cant do that until I confirm above in step 1 

step 3) I want to make it so that it doesn doe the other ones over again I can just index those out 
since I am making one giant H5 file, I can preallocate it and copy all the good stuff to that one, perfect. 

"""



from natsort import natsorted, ns

d = '/Users/phil/Dropbox/HIRES_LAB/GitHub/whacc/whacc/whacc_data/feature_data/feature_data_dict.pkl'
d = utils.load_obj(d)
utils.get_dict_info(d)
x = np.asarray(d['full_feature_names'])
features_used_name  = x[d['final_selected_features_bool']]
x = np.unique(features_used_name)
x = natsorted(x, alg=ns.REAL)
for k in x:
    print(k)
    print(np.sum(k==features_used_name))


tmp1 = utils.lister_it(x, 'TOTAL')
for k in tmp1:
    print(k)

tmp2 = utils.lister_it(x, remove_string='TOTAL')
for k in tmp2:
    print(k)


for k in tmp1:
    kk = k.replace('FD_TOTAL_std_1_of_', '')
    kk in tmp2

"""
find the max I need to make
"""
# I need to make at least 10 total feature

tmp3 = []
for k in tmp1:
    tmp3.append(k.replace('FD_TOTAL_std_1_of_', ''))


for k in tmp2:
    if k not in tmp3:
        print(k)
        print(np.sum(k==features_used_name))
    else:
        print('++++++++++++++++++++')
        print(k)
        print(np.sum(k==features_used_name))
        print('____________________')

for k in tmp2:
    if k in tmp3:
        print(k)


"""
generate the total features first



test when ind is 1 
shape is none
"""
d['feature_list_unaltered']




from whacc import utils
import numpy as np
from whacc.retrain_LGBM import retrain_LGBM

bd = '/Users/phil/Desktop/untitled folder/'
labels_key = 'labels'
h5_list = utils.get_files(bd, '*.h5')
tvt_x, tvt_y, tvt_fn, tvt_w = utils.load_training_and_curated_data(h5_list, labels_key)


model_base_dir = '/Users/phil/Desktop/tmp_mod_test/'
study_name = 'my_custom_optuna_models_test_V2'

#VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
load_optuna_if_exists = False # change to true if continuing training<<<<<<<<<<<<<<<<
#AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA

rm = retrain_LGBM(model_base_dir, study_name, tvt_x, tvt_y, tvt_fn, tvt_w, load_optuna_if_exists=load_optuna_if_exists)

rm.GLOBALS['num_optuna_trials'] = 20  ########  20  3
rm.GLOBALS['early_stopping_rounds'] = 100  ########  100 10
rm.GLOBALS['num_iterations'] = 1000 ########  10000 5




rm.train_model()


# after doubling
tot = 526222+147854+70924
526222/tot, 147854/tot, 70924/tot
#percent each of total
(0.7063382550335571, 0.19846174496644295, 0.0952)
# percent touch
0.43341783505820736
0.4155315378684378
0.3958039591675596



tmp1 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/all_pred_dict.pkl'
tmp1 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/all_pred_dict3.pkl'
tmp1 = utils.load_obj(tmp1)

utils.info(tmp1)

all_sizes = []
for k in tmp1:
    all_sizes.append(len(tmp1[k]))

# all validation set sizes for 10 models.... ([212740, 212746, 212754, 212764, 212765, 212821, 212823, 212859, 212866, 212867])
#  toal of 2,128,005
keys = list(tmp1.keys())
tmp2 = tmp1[keys[-1]]

import matplotlib.pyplot as plt
plt.plot(tmp2[-4000:])
# import matplotlib.pyplot as plt
# for k in tvt_y:
#     print(np.mean(k))
# plt.ylim(0, 1)






from setuptools import setup
from setuptools import find_packages
import os
from whacc import utils

with open("README.md", "r") as fh:
    long_description = fh.read()


def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            if filename[0] != '.':
                paths.append(os.path.join('..', path, filename))
    return paths


include_files = package_files(utils.get_whacc_path())  # need to verify this will work on all machines... using utils
# before WhACC is installed??????

remove_list = ['final_model/final_resnet50V2_full_model',
               'whacc_data/training_data',
               'final_model/extra_LGBM_models']
include_files = list(utils.lister_it(include_files, remove_string=remove_list))

setup(name='whacc',
      version='1.4.03',
      author="Phillip Maire",
      license='MIT',
      description='Automatic and customizable pipeline for creating a CNN + light GBM model to predict whiskers contacting objects',
      packages=find_packages(),
      author_email='phillip.maire@gmail.com',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/hireslab/whacc",
      zip_safe=False,
      classifiers=["Programming Language :: Python :: 3",
                   "License :: OSI Approved :: MIT License",
                   "Operating System :: OS Independent"],
      python_requires='>=3.6',
      install_requires=["natsort==7.1.1"],
      package_data={'': include_files},
      # exclude_package_data=exclude_package_data,
      # package_data={'whacc': ['whacc_data/*',
      #                         'whacc/whacc_data/feature_data/*',
      #                         'whacc/whacc_data/feature_data/*.pkl',
      #                         'whacc/whacc_data/*.png',
      #                         'whacc_data/final_model/final_resnet50V2_full_model/variables/*',
      #                         'whacc_data/final_model/final_resnet50V2_full_model/assets/*',
      #                         'whacc_data/final_model/final_resnet50V2_full_model/*',
      #                         'whacc_data/final_model/*',
      #                         'whacc/whacc_data/final_model/final_resnet50V2_full_model/*pb',
      #                         'whacc/whacc_data/final_model/final_resnet50V2_full_model/variables/*.index',
      #                         'whacc/whacc_data/final_model/final_resnet50V2_full_model/variables/*.data-00000-of-00001']},
      include_package_data=True
      )



from whacc import utils, transfer_learning

import matplotlib.pyplot as plt
from tensorflow.keras.layers import Dense, Dropout
from tensorflow.keras.models import Model
from tensorflow import keras

from sklearn.utils import class_weight

import os
import copy
import numpy as np
from tensorflow.keras import applications
from pathlib import Path

import shutil
import zipfile
from datetime import datetime
import pytz
import json
import tensorflow as tf

def load_final_model_HEAD():
    mod_path = utils.get_whacc_path() + '/whacc_data/final_model/final_resnet50V2_HEAD/final_model_HEAD'
    mod = tf.keras.models.load_model(mod_path)
    mod.compile()

    # mod_path = utils.get_whacc_path() + '/whacc_data/final_model/final_resnet50V2_HEAD/final_CNN_model_head.pkl'
    # mod = utils.load_obj(mod_path)
    # mod.compile()
    return mod
def load_final_model(keep_head = False):
    mod_path = utils.get_whacc_path() + '/whacc_data/final_model/final_resnet50V2_full_model'
    if not os.path.isdir(mod_path):
        print('Auto downloading resnet model, this will only run once')
        utils.download_resnet_model()
    mod = keras.models.load_model(mod_path)
    if keep_head:
        return mod
    else:
        num_layers_in_base_model = 190 #used originally to chop different base models kept for posterity
        mod = Model(mod.input, mod.layers[num_layers_in_base_model +1].output) # cut off head
        mod.compile()
        return mod

def load_final_light_GBM():
    # mod_path = utils.get_whacc_path() + '/whacc_data/final_model/light_GBM_model/mod_67_with_threshold.pkl'
    mod_path = utils.get_whacc_path() + '/whacc_data/final_model/light_GBM_model/FINAL_WHACC_MODEL_016_model.pkl'
    mod = utils.load_obj(mod_path)
    return mod

def build_model(info_dict, labels, model_name_str, base_learning_rate=0.00001, dropout_val=None, class_numbers=None,
                IMG_SIZE=96):
    if class_numbers is None:
        class_numbers = np.unique(labels)
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
    class_weights = {i: rebalance[i] for i in class_numbers}
    wrap_vars_list = ['class_numbers',
                      'num_classes',
                      'base_learning_rate',
                      'model_name_str',
                      'IMG_SIZE',
                      'dropout_val']
    for k in wrap_vars_list:
        info_dict[k] = eval(k)
    info_dict['class_weights'] = str(class_weights)
    return model, class_weights, info_dict


def label_naming_shorthand_dict(name_key=None):
    label_naming_shorthand_dict = {
        '[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)': 'on-off_set_and_one_after',
        '[0, 1, 2, 3]- (no touch, touch, onset, offset': 'on-off_set',
        '[0, 1, 2]- (no event, onset, offset)': 'only_on-off_set',
        '[0, 1]- (no touch, touch)': 'regular',
        '[0, 1]- (not offset, offset)': 'only_offset',
        '[0, 1]- (not onset, onset)': 'only_onset',
        '[0, 1, 2, 3]- (no touch, touch, one after onset, offset)': 'overlap_whisker_on-off'}
    if name_key is None:
        return label_naming_shorthand_dict
    else:
        return label_naming_shorthand_dict[name_key]


def info_dict_wrapper(info_dict, local_dict):
    for k in local_dict.keys():
        info_dict[k] = local_dict[k]
    return info_dict


def make_initial_folder(all_models_directory, unique_h5_train_val_dir):
    single_frame_dir = all_models_directory + os.sep + unique_h5_train_val_dir + os.sep + 'data' + os.sep + 'single_frame'
    Path(single_frame_dir).mkdir(parents=True, exist_ok=True)
    return locals()
    # return single_frame_dir


def get_automated_model_info(BASE_H5, image_source_h5_directory_ending, test_data_dir):
    tz = pytz.timezone('America/Los_Angeles')
    loc_dt = pytz.utc.localize(datetime.utcnow())
    LA_TIME = loc_dt.astimezone(tz)
    todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")
    del tz
    del loc_dt
    del LA_TIME
    a = os.sep
    base_data_dir = BASE_H5 + a + "data" + a
    base_dir_all_h5s = BASE_H5 + a + 'data' + a + 'single_frame' + a
    data_dir = base_data_dir + image_source_h5_directory_ending
    print('\nFOR IMAGES, 0 is train set, 1 is val set')
    print(data_dir)
    image_h5_list = utils.get_h5s(data_dir)
    h5_train = image_h5_list[0]
    h5_val = image_h5_list[1]
    labels_dir = base_data_dir + a + "ALT_LABELS" + a
    print('\nFOR LABELS,0 is train set, 1 is val set')
    label_h5_list = utils.get_h5s(labels_dir)
    print('\nSelect from the following label structures...')
    label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)
    h5_test_labels = utils.get_h5s(test_data_dir + a + "ALT_LABELS" + a, print_h5_list=False)[0]
    h5_test = utils.get_h5s(test_data_dir + a + image_source_h5_directory_ending + a, print_h5_list=False)[0]
    return locals()


def copy_over_new_labels(label_key_name, image_h5_list, label_h5_list):
    label_key_shorthand = label_naming_shorthand_dict(label_key_name)
    for img_src, lab_src in zip(image_h5_list, label_h5_list):
        utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')
    return locals()


# get list of pre trained models to choose from
def get_keras_model_names():
    names_, types_ = utils.get_class_info(applications, return_name_and_type=True)
    model_names = np.asarray(names_)['function' == np.asarray(types_)]
    utils.print_list_with_inds(model_names)
    return model_names


def make_model_save_directory(info_dict, make_folder=True):
    naming_list = ['model_name_str', 'image_source_h5_directory_ending', 'label_key_shorthand', 'todays_version']
    model_save_dir: str = copy.deepcopy(info_dict['BASE_H5'])
    for k in naming_list:
        model_save_dir += os.sep + info_dict[k] + os.sep
    info_dict['model_save_dir'] = model_save_dir
    info_dict['model_save_dir_checkpoints'] = model_save_dir + os.sep + 'checkpoints'
    if make_folder:
        Path(info_dict['model_save_dir_checkpoints']).mkdir(parents=True, exist_ok=True)
    return model_save_dir


def basic_callbacks(save_checkpoint_filepath, monitor='val_loss', patience=10,
                    save_best_only=False, save_weights_only=True, save_freq="epoch", period=1):
    callbacks = []
    callbacks.append(keras.callbacks.EarlyStopping(monitor=monitor, patience=patience))
    add_path_name = "{loss:.8f}_{epoch:04d}_cp.hdf5"
    add_path_name = "{epoch:04d}_cp.hdf5"
    callbacks.append(keras.callbacks.ModelCheckpoint(
        save_checkpoint_filepath + os.sep + add_path_name,
        monitor=monitor,
        save_best_only=save_best_only,
        save_weights_only=save_weights_only,
        save_freq=save_freq,
        period=period))
    return callbacks


def unzip_and_place_h5s(bd, do_delete_zips=False):
    bd2 = ''.join(bd.split('gdrive/My Drive'))
    shutil.copytree(bd, bd2)
    a = utils.get_files(bd2, '*.zip')
    for k in a:
        with zipfile.ZipFile(k, 'r') as zip_ref:
            zip_ref.extractall(os.path.dirname(k))
        if do_delete_zips:
            os.remove(k)


def change_to_local_dir(bd):
    bd2 = ''.join(bd.split('gdrive/My Drive'))
    return bd2


def change_to_gdrive_dir(bd):
    bd2 = '/content/gdrive/My Drive/'.join(bd.split('/content/'))
    return bd2


def pack_training_info(save_and_plot_history_var):
    training_info = dict()
    for k in ['L', 'all_logs', 'all_colors', 'logs_names', 'markers', 'matching_inds']:
        training_info[k] = eval('save_and_plot_history_var.' + k)
    for k in training_info.keys():
        if 'numpy.ndarray' in str(type(training_info[k])):
            training_info[k] = training_info[k].tolist()
    return training_info


def save_info_dict(info_dict_tmp):
    for k in info_dict_tmp.keys():
        if 'numpy.ndarray' in str(type(info_dict_tmp[k])):
            info_dict_tmp[k] = info_dict_tmp[k].tolist()
    with open(info_dict_tmp['model_save_dir'] + 'info_dict' + '.json', 'w') as f:
        json.dump(info_dict_tmp, f)


def foo_save_and_plot(training_info, save_and_plot_history_1, save_loc):
    text_str = transfer_learning.make_text_for_fig(training_info)
    training_info = transfer_learning.pack_training_info(training_info, save_and_plot_history_1)
    # save_num = foo_get_next_save_num(save_loc)

    # make figure and save
    transfer_learning.plot_train_hist(training_info, [1], [.9, 1], text_str)
    plt.savefig(save_loc + 'mod_test_fig_ACC' + '.png', bbox_inches="tight")
    transfer_learning.plot_train_hist(training_info, [0], [0, 0.25], text_str)
    plt.savefig(save_loc + 'mod_test_fig_LOSS' + '.png', bbox_inches="tight")

    # save training info
    with open(save_loc + 'model_eval_each_epoch' + '.json', 'w') as f:
        json.dump(training_info, f)


def re_build_model(model_name_str, class_numbers, base_learning_rate=0.00001, dropout_val=None, IMG_SIZE=96,
                   labels=None):
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    if labels is not None:
        rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
        class_weights = {i: rebalance[i] for i in class_numbers}
        wrap_vars_list = ['class_numbers',
                          'num_classes',
                          'base_learning_rate',
                          'model_name_str',
                          'IMG_SIZE',
                          'dropout_val']
        return model, class_weights
    else:
        return model


def reload_info_dict(base_dir):
    info_dict_path = utils.get_files(base_dir, '*info_dict.json')[0]
    with open(info_dict_path, 'r') as f:
        D = json.load(f)
    # replace the directory to the main directory
    x = 'model_testing'
    old_base_dir = ''
    for k in D.keys():
        try:
            if x in D[k]:
                D[k] = base_dir.split(x)[0] + x + D[k].split(x)[-1]
                old_base_dir = D[k].split(x)[0]
        except:
            pass
    D['old_base_dir'] = old_base_dir
    D['checkpoints'] = utils.get_files(D['model_save_dir_checkpoints'], '*hdf5')
    return D


def load_model_data(all_models_directory):
    data_files = utils.get_files(all_models_directory, '*model_eval_each_epoch.json')
    to_split = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/'
    all_data = []
    for i, k in enumerate(data_files):
        if '/small_h5s/' not in k:
            with open(k, 'r') as f:
                a = json.load(f)
            a['all_logs'] = np.asarray(a['all_logs'])
            new_log_names = []
            for k2 in a['logs_names']:
                new_log_names.append(k2.split('bool_')[-1])
            a['logs_names'] = new_log_names
            a['logs_names'] = np.asarray(a['logs_names'])

            k.split(to_split)[-1]
            a['full_name'] = '__'.join(' '.join(k.split(to_split)[-1].split('/2021')[0].split('_')).split('/'))
            a['dir'] = os.path.dirname(k)
            all_data.append(a)
    return all_data
# def reload_info(base_dir):
#   s = os.sep
#   b = base_dir.split('all_models'+s)[-1].split(s)
#   reload_dict = {'data_type':b[0],
#                 'model_name':b[1],
#                 'img_type':b[2],
#                 'label_name_shorthand':b[3],
#                 'date_string':b[4]}
#
#   label_name_dict = label_naming_shorthand_dict()
#   for k in label_name_dict.keys():
#     if reload_dict['label_name_shorthand'] == label_name_dict[k]:
#       reload_dict['label_type'] = k
#       class_numbers = eval(k.split('- ')[0])
#   reload_dict['class_numbers'] = class_numbers
#   return reload_dict



from whacc import PoleTracking, utils
import whacc
import glob
import os
import matplotlib.pyplot as plt
from whacc.touch_curation_GUI import touch_gui

# this will use the pole tracker on all subdirectories, if you want to do it on one directory
# just set ... video_directory = ['your/full.directory/here/']
mp4_path = '/Users/phil/Desktop/pipeline_test/'
search_term = '*.mp4'
folders_with_MP4s = whacc.utils.recursive_dir_finder(mp4_path, search_term)
print(folders_with_MP4s, sep='\n')

PT = dict()
for i, video_directory in enumerate(folders_with_MP4s):
    template_img_full_name = glob.glob(video_directory + os.path.sep + '*template_img.png')
    if len(template_img_full_name) == 1:  # if the template already exists
        PT[i] = PoleTracking(video_directory=video_directory, template_png_full_name=template_img_full_name[0])
    else:
        PT[i] = PoleTracking(video_directory=video_directory)  # create the 'PoleTracking' class
        PT[i].cut_out_pole_template(crop_size=[61, 61], frame_num=2000, file_ind=2)  # user cut out template image
        PT[i].save_template_img(cust_save_dir=PT[i].video_directory)  # save the template image

for i in PT: #plot image and display trial numbers extract to ensure they match
    PT_tmp = PT[i]
    PT_tmp.get_trial_and_file_names(print_them=True, num_to_print=5)
    PT_tmp.save_template_img(cust_save_dir=PT_tmp.video_directory)
    plt.figure()
    plt.title(PT_tmp.video_directory)
    plt.imshow(PT_tmp.template_image)

for i in PT:
    output_h5 = PT[i].track_all_and_save()

f = '/Users/phil/Desktop/pipeline_test/AH0407x160609.h5'
f2 = f.replace('.h5', '_3lag.h5')
utils.convert_to_3lag(f, f2)
utils.print_h5_keys(f2)
utils.print_h5_keys(f)


f = '/Users/phil/Desktop/pipeline_test/AH0407x160609.h5'
f2 = f.replace('.h5', '_3lag.h5')
utils.stack_lag_h5_maker(f, f2, buffer=2, shift_to_the_right_by=0) # dont change these setting these are for making 3lag, make simple function for this

# if you want to hand curate you can use the GUI, you can also scan through your images.

touch_gui(f2, 'labels', 'labels')

touch_gui('/Users/phil/Dropbox/HIRES_LAB/TL_data_FINAL_CURATED/TL_DATA_AH0706x171107_final_combined.h5', 'labels')



import shutil

import matplotlib.pyplot as plt
from tqdm.auto import tqdm
from imgaug import augmenters as iaa  # optional program to further augment data

from whacc import utils
import numpy as np
from whacc import image_tools
from natsort import natsorted, ns
import pickle
import pandas as pd
import os
import copy
import seaborn as sns
from keras.preprocessing.image import ImageDataGenerator
import h5py

from whacc import utils
import h5py
import matplotlib.pyplot as plt
import numpy as np
from pathlib import Path
def foo_save(name_in, data):
    tmp1 = os.path.dirname(name_in)
    Path(tmp1).mkdir(parents=True, exist_ok=True)
    np.save(name_in, data)
from tqdm.contrib import tzip
"""
basically most of what I did is on colab 
code is kinda all over the place 
1) I took the 3lag full session H5 datasets and save frame nums, labels and frame nums to s folder structure as .NPY files
2) I then augmented the full session (didn't cut session off here because I want the final features to have little or no NAN values when shifitng and rolling 
3) in colab I then converted those full sessions to 2048 using GPU for all session 8 normal and 80 augmented 
4) in colab with CPU sessions I generated the final 2105 features
5) no I have to save the raw frame number   

"""
"""
################################################################################################
make the majority labels and the individual labels
################################################################################################
"""
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
image_dir = bd + '/images/'
label_dir = bd + '/labels/'
image_files = natsorted(utils.get_files(image_dir, '*.npy'), alg=ns.REAL)

all_h5s = utils.get_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/Colab data/curation_for_auto_curator/finished_contacts/')
h_cont, list_of_uniq_files, curator_names = utils._get_human_contacts_(all_h5s, return_curator_names = True)


for i, k in enumerate(list_of_uniq_files): # make sure they match
    to_end = len(k[:-4])
    print(k[:to_end])
    kk = os.path.basename(image_files[i])
    print(kk[:to_end])
    assert kk[:to_end] == k[:to_end], 'files do not match'
    print('___')

for i, k in enumerate(h_cont):
    majority_labels = 1*(np.mean(k, axis=0)>.5)
    kk = os.path.basename(image_files[i])[:-4]
    foo_save(label_dir+'/'+kk, majority_labels)

for ii, name in enumerate(curator_names):
    for i, k in enumerate(h_cont):
        kk = os.path.basename(image_files[i])[:-4]
        labels_2_save = k[ii]
        foo_save(bd+'/individual_labels/'+name+'/'+kk, labels_2_save)


"""
################################################################################################
Save images and augmented images as numpy in different folders
################################################################################################
"""

# h5_meta_data = '/Users/phil/Dropbox/Colab data/H5_data/regular/AH0407_160613_JC1003_AAAC_regular.h5'
# h5_frames = '/Users/phil/Dropbox/Colab data/H5_data/3lag/AH0407_160613_JC1003_AAAC_3lag.h5'
# h5_frames = '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/AH0407_160613_JC1003_AAAC_ALT_LABELS.h5'

border = 80
d_list = ['/Users/phil/Dropbox/Colab data/H5_data/regular/',
          '/Users/phil/Dropbox/Colab data/H5_data/3lag/',
          '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/',
          '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS']

h5_meta_images_labels = []
for k in d_list: # just save all the dat to numpy in different folders
    sorted_files = natsorted(utils.get_h5s(k), alg=ns.REAL)
    h5_meta_images_labels.append(sorted_files)

cnt = 0
for h5_meta_data, h5_frames, h5_labels2, h5_labels in tqdm(np.asarray(h5_meta_images_labels).T):
    cnt+=1
    print(cnt)
    if cnt >=7:
        h5_meta_data, h5_frames, h5_labels2, h5_labels = str(h5_meta_data), str(h5_frames), str(h5_labels), str(h5_labels2)

        y = image_tools.get_h5_key_and_concatenate(h5_labels, '[0, 1]- (no touch, touch)')
        OG_frame_nums = image_tools.get_h5_key_and_concatenate(h5_meta_data, 'frame_nums')
        images = image_tools.get_h5_key_and_concatenate(h5_frames, 'images')

        end_name = os.path.basename(h5_frames)[:-3]
        save_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
        name = save_dir + '/images/' + end_name
        if not os.path.isfile(name):
            foo_save(name, images)

            name = save_dir + '/frame_nums/' + end_name
            foo_save(name, OG_frame_nums)

            # name = save_dir + '/labels/' + end_name ##### WRONGGGGGGGG
            # foo_save(name, y)

        for aug_num in tqdm(range(10)): # Augment images
            name = save_dir + '/images_AUG_'+ str(aug_num) +'/' + end_name
            if not os.path.isfile(name):
                datagen = ImageDataGenerator(rotation_range=360,  #
                                                width_shift_range=.1,  #
                                                height_shift_range=.1,  #
                                                shear_range=.00,  #
                                                zoom_range=.25,
                                                brightness_range=[0.2, 1.2])  #
                gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
                num_aug = 1


                aug_img_stack = []
                for image, label in zip(images, y):
                    aug_img, _ = image_tools.augment_helper(datagen, num_aug, 0, image, label)
                    aug_img = gaussian_noise.augment_images(aug_img) # optional
                    aug_img_stack.append(aug_img)

                aug_img_stack = np.squeeze(np.asarray(aug_img_stack))
                foo_save(name, aug_img_stack)
        del images



# RESNET_MODEL = model_maker.load_final_model()
# features = model.predict(x)
"""
################################################################################################
################################################################################################
these are all the inds to the good videos so I can multiply the labels by this and it will 
cancel out all the bad videos by setting the labels to 0 so border extraction will not happen on them!
################################################################################################
################################################################################################
"""


from scipy import stats
ind_naming = '/keep_inds_no_skip_frames/'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/frame_nums'
bd2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
all_fn = []
sorted_files = natsorted(utils.get_files(bd, '*.npy'), alg=ns.REAL)
for k in sorted_files:
    fn = np.load(k, allow_pickle=True)
    all_fn.append(fn)
    mode = stats.mode(fn)[0][0]
    good_vids = fn==mode
    label_multiply_by = []
    for ii, num_frames in enumerate(fn):
        if good_vids[ii]:
            label_multiply_by.append([1] * num_frames)
        else:
            label_multiply_by.append([0] * num_frames)
    label_multiply_by = np.concatenate(label_multiply_by)
    save_name = bd2 +ind_naming+ '/' + os.path.basename(k)
    if '________________________AH1120_200322' in os.path.basename(k): # remove samsons bad data
        print('AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    ')
        label_multiply_by = label_multiply_by*0 # remove all of one of samson's datasets
    foo_save(save_name, label_multiply_by)




"""
################################################################################################
################################################################################################
these are all the inds to the good videos so I can multiply the labels by this and it will 
cancel out all the bad videos by setting the labels to 0 so border extraction will not happen on them!
################################################################################################
################################################################################################
"""

from scipy import stats
ind_naming = '/keep_inds_no_skip_frames/'
ind_naming = 'keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/frame_nums'
bd2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
all_fn = []
sorted_files = natsorted(utils.get_files(bd, '*.npy'), alg=ns.REAL)
for k in sorted_files:
    fn = np.load(k, allow_pickle=True)
    all_fn.append(fn)
    mode = stats.mode(fn)[0][0]
    good_vids = fn==mode
    label_multiply_by = []
    for ii, num_frames in enumerate(fn):
        if good_vids[ii]:
            label_multiply_by.append([1] * num_frames)
        else:
            label_multiply_by.append([0] * num_frames)
    label_multiply_by = np.concatenate(label_multiply_by)
    save_name = bd2 +ind_naming+ '/' + os.path.basename(k)
    if 'AH1120_200322' in os.path.basename(k): # remove samsons bad data
        print('AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    ')
        label_multiply_by = label_multiply_by*0 # remove all of one of samson's datasets

    foo_save(save_name, label_multiply_by)

"""
######################## make inds for training data for comparing to CNN ##############################################
"""
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/80_border/T_V_TS_set_inds/'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_skip_frames/80_border/T_V_TS_set_inds/'
for k in utils.get_files(bd, '*.npy'):
    inds = np.load(k, allow_pickle=True)
    inds = np.sort(np.concatenate(inds))

"""
"""
# bd = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/3lag/'
# for k in utils.get_h5s(bd):
#     utils.print_h5_keys(k)
#
#
#
# h5 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/3lag/val_3lag.h5'
# utils.print_h5_keys(h5)

"""
################################################################################################
################################################################################################
make the majority labels and the individual labels
################################################################################################
################################################################################################
################################################################################################
"""

border = 80
second_border = 3
split_ratio = [7, 2, 1]
label_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/labels'
regular_dir = '/Users/phil/Dropbox/Colab data/H5_data/regular/'

bd_good_vids = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/'
bd_good_vids = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_CNN_compare/'
# bd_good_vids = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_skip_frames/'
# bd_good_vids = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_cnn_compare/'

bd_80 = bd_good_vids+'/80_border/'
bd_3 = bd_good_vids+'/3_border/'
# utils.rmtree(bd_3)
# utils.rmtree(bd_80)
"""
get split ratio to copy it 
use it for the first 4 videos and then done 

full set multiply by the 80 border for the train and Val 
"""
split_ratio_list = [[.7, .3], [.7, .3], [.7, .3], [.7, .3], [0,0,1], [0,0,1], [0,0,1], [0,0,1]]
label_files = natsorted(utils.get_files(label_dir, '*.npy'), alg=ns.REAL)
h5_meta_data_files = natsorted(utils.get_files(regular_dir, '*.h5'), alg=ns.REAL)

for i, (label_f, h5_meta_data) in enumerate(zip(label_files, h5_meta_data_files)):
    split_ratio = split_ratio_list[i]
    good_vid_inds = np.load(bd_good_vids+os.path.basename(label_f), allow_pickle=True)
    base_name = os.path.basename(label_f)
    labels = np.load(label_f, allow_pickle=True)
    labels = good_vid_inds*labels # this will remove all the bad videos by making the labels == 0

    if not np.all(labels==0):
        """
        label_f had continious variable, need to source the human data directly and stor ein the folders under the names
        then make a final npy ile with all labels 
        """
        b = utils.inds_around_inds(labels, border * 2 + 1)
        group_inds, _ = utils.group_consecutives(b)
        new_frame_nums = []
        for tmp2 in group_inds:
            new_frame_nums.append(len(tmp2))

        OG_frame_nums = image_tools.get_h5_key_and_concatenate(h5_meta_data, 'frame_nums')
        OG_frame_nums_cumulative = np.cumsum(OG_frame_nums)

        trial_ind_1 = []
        trial_ind_2 = []
        for k in group_inds:
            trial_ind_1.append(np.sum(k[0]>=OG_frame_nums_cumulative))
            trial_ind_2.append(np.sum(k[0]>=OG_frame_nums_cumulative))
        assert np.all(trial_ind_2 == trial_ind_1), 'there are overlapping images from one video to the next, which should be ' \
                                                   'impossible unless the pole stays up between trials '

        np.random.seed(0)
        tmp_sets = utils.split_list(group_inds, split_ratio)
        if len(tmp_sets)==2:
            tmp_sets.append([])

        T_V_TS_sets = []
        frame_nums_set = []
        labels_80 = []
        # labels = np.asarray(labels)
        for k in tmp_sets:
            if len(k) == 0:
                T_V_TS_sets.append(sorted(k))
            else:
                T_V_TS_sets.append(sorted(np.concatenate(k)))
            frame_nums_set.append(len(k))
            labels_80.append(labels[T_V_TS_sets[-1]])
        foo_save(bd_80 + '/T_V_TS_set_inds_CNN_COMPARE/'+base_name, list(T_V_TS_sets))
        # foo_save(bd_80 + '/labels/'+ base_name, labels_80)

        # T_V_TS_sets_3_border = []
        # labels_3 = []
        # for k in T_V_TS_sets:
        #     tmp_labels = labels[k]
        #     b = utils.inds_around_inds(tmp_labels, second_border * 2 + 1)
        #     group_inds, _ = utils.group_consecutives(b)
        #     border_3_inds = np.asarray(k)[np.concatenate(group_inds)]
        #     T_V_TS_sets_3_border.append(border_3_inds)
        #     labels_3.append(labels[T_V_TS_sets_3_border[-1]])
        #
        # # foo_save(bd_3 + '/T_V_TS_set_inds_cnn_compare/'+ base_name, T_V_TS_sets_3_border)
        # foo_save(bd_3 + '/T_V_TS_set_inds/'+ base_name, T_V_TS_sets_3_border)
        # foo_save(bd_3 + '/labels/'+ base_name, labels_3)
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_CNN_compare/80_border/T_V_TS_set_inds_CNN_COMPARE'

for k in utils.get_files(bd, '*.npy'):
    tmp1 = np.load(k, allow_pickle=True)
    print(len(tmp1))
    asdf


bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/80_border/'
ind_dir = bd+'/T_V_TS_set_inds/'
save_data_dir = bd + '/final_2105/'
feature_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/final_2105/'
feature_files = utils.sort(utils.get_files(feature_dir, '*.npy'))
feature_files_base_names = np.asarray([os.path.basename(k) for k in feature_files])
for k in tqdm(utils.sort(utils.get_files(ind_dir, '*.npy'))):
    tmp1 = np.load(k, allow_pickle=True)

    basename_inds = os.path.basename(k)
    iii = np.where(basename_inds == feature_files_base_names)[0][0]
    data_file = feature_files[iii]


    # label_file = utils.norm_path(save_data_dir+os.path.basename(data_file), '/').split('/')
    # label_file[-2] = 'labels'
    # label_file = '/' + '/'.join(label_file)
    # labels = np.load(label_file, allow_pickle=True)

    base_name = '_'+os.path.basename(data_file)
    run_it = False
    for (inds, save_name) in zip(tmp1, ['train', 'val', 'test']):
        fn = save_data_dir+save_name+base_name
        # labels_name = os.path.dirname(fn)+'_labels/'+os.path.basename(fn)
        # x = labels[inds]
        # foo_save(labels_name, x)
        if os.path.isfile(fn):
            run_it = True
    if run_it:
        final_2105 = np.load(data_file, allow_pickle=True)
        for (inds, save_name) in zip(tmp1, ['train', 'val', 'test']):
            x = final_2105[inds, :]
            foo_save(save_data_dir+save_name+base_name, x)
            #make new folder

"""
need to add the labels here too

"""



bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/3_border/'
ind_dir = bd+'/T_V_TS_set_inds/'

for folder_num in range(10):
    feature_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/final_2105_AUG_'+str(folder_num)+os.sep
    save_data_dir = bd + '/final_2105'+'_AUG_'+str(folder_num)+os.sep
    feature_files = utils.sort(utils.get_files(feature_dir, '*.npy'))
    feature_files_base_names = np.asarray([os.path.basename(k) for k in feature_files])
    for k in tqdm(utils.sort(utils.get_files(ind_dir, '*.npy'))):
        basename_inds = os.path.basename(k)
        iii = np.where(basename_inds == feature_files_base_names)[0][0]
        data_file = feature_files[iii]
        base_name = '_'+os.path.basename(data_file)
        run_it = False

        for save_name in ['train', 'val', 'test']:
            fn = save_data_dir+save_name+base_name
            if not os.path.isfile(fn):
                run_it = True

        if run_it:
            tmp1 = np.load(k, allow_pickle=True)
            final_2105 = np.load(data_file, allow_pickle=True)
            for (inds, save_name) in zip(tmp1, ['train', 'val', 'test']):
                x = final_2105[inds, :]
                foo_save(save_data_dir+save_name+base_name, x)


        #
        # base_name = '_'+os.path.basename(data_file)
        # for (inds, save_name) in zip(tmp1, ['train', 'val', 'test']):
        #     x = final_2105[inds, :]
        #     foo_save(save_data_dir+save_name+base_name, x)
        # if run_it:
        #     final_2105 = np.load(data_file, allow_pickle=True)
        #


"""
################################################################################################
################################################################################################
################################################################################################
################################################################################################
generate the final datasets 
need to account for missing frame datasets and samsons dataset 

shit I realized i need to remove the bad ones and then chose the final indexes otherwise it can reduce the 
total number of in any particular set 


ok so to re
"""
h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/80_border/T_V_TS_set_inds/AH0407_160613_JC1003_AAAC_3lag.npy'
h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/80_border/T_V_TS_set_inds/AH0407_160613_JC1003_AAAC_3lag.npy'
h5_list = utils.get_h5s('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/80_border/T_V_TS_set_inds')
utils.print_h5_keys(h5)
np_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/80_border/T_V_TS_set_inds/AH0407_160613_JC1003_AAAC_3lag.npy'
tmp1 = np.load(np_in, allow_pickle=True)

tmp1[0]


################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################

def foo_save(name_in, data):
        tmp1 = os.path.dirname(name_in)
        Path(tmp1).mkdir(parents=True, exist_ok=True)
        np.save(name_in, data)
# h5_meta_data = '/Users/phil/Dropbox/Colab data/H5_data/regular/AH0407_160613_JC1003_AAAC_regular.h5'
# h5_frames = '/Users/phil/Dropbox/Colab data/H5_data/3lag/AH0407_160613_JC1003_AAAC_3lag.h5'
# h5_frames = '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/AH0407_160613_JC1003_AAAC_ALT_LABELS.h5'

border = 80
d_list = ['/Users/phil/Dropbox/Colab data/H5_data/regular/',
          '/Users/phil/Dropbox/Colab data/H5_data/3lag/',
          '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/',
          '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS']

h5_meta_images_labels = []
for k in d_list:
    sorted_files = natsorted(utils.get_h5s(k), alg=ns.REAL)
    h5_meta_images_labels.append(sorted_files)

for h5_meta_data, h5_frames, h5_labels2, h5_labels in tqdm(np.asarray(h5_meta_images_labels).T):
    # for k in [h5_meta_data, h5_frames, h5_labels2, h5_labels]:
    #     print(os.path.basename(k))
    # print('______')
    h5_meta_data, h5_frames, h5_labels2, h5_labels = str(h5_meta_data), str(h5_frames), str(h5_labels), str(h5_labels2)

    # get the 80 border indices and the 3 border indices
    y = image_tools.get_h5_key_and_concatenate(h5_labels, '[0, 1]- (no touch, touch)')


    # y = np.zeros(4001)
    # y[2000] = 1
    OG_frame_nums = image_tools.get_h5_key_and_concatenate(h5_meta_data, 'frame_nums')
    b = utils.inds_around_inds(y, border * 2 + 1)
    group_inds, result_ind = utils.group_consecutives(b)
    new_frame_nums = []
    for tmp2 in group_inds:
        new_frame_nums.append(len(tmp2))

    OG_frame_nums_cumulative = np.cumsum(OG_frame_nums)
    trial_ind_1 = []
    trial_ind_2 = []
    for k in group_inds:
        trial_ind_1.append(np.sum(k[0]>=OG_frame_nums_cumulative))
        trial_ind_2.append(np.sum(k[0]>=OG_frame_nums_cumulative))
    assert np.all(trial_ind_2 == trial_ind_1), 'there are overlapping images from one video to the next'

    images = image_tools.get_h5_key_and_concatenate(h5_frames, 'images')
    extracted_images = images[np.concatenate(group_inds), :, :, :]

    del images
    labels = np.asarray(y)[np.concatenate(group_inds)]
    end_name = os.path.basename(h5_frames)[:-3]
    name = '/Users/phil/Dropbox/Colab data/H5_data/3lag_80_border_numpy/images/' + end_name
    foo_save(name, extracted_images)
    name = '/Users/phil/Dropbox/Colab data/H5_data/3lag_80_border_numpy/80_border_inds/' + end_name
    foo_save(name, group_inds)
    name = '/Users/phil/Dropbox/Colab data/H5_data/3lag_80_border_numpy/frame_nums/' + end_name
    foo_save(name, new_frame_nums)
    # name = '/Users/phil/Dropbox/Colab data/H5_data/3lag_80_border_numpy/labels/' + end_name # WRONGGGGGG
    # foo_save(name, labels)
    del extracted_images

    #
    # for aug_num in range(10):
    #     datagen = ImageDataGenerator(rotation_range=360,  #
    #                                     width_shift_range=.1,  #
    #                                     height_shift_range=.1,  #
    #                                     shear_range=.00,  #
    #                                     zoom_range=.25,
    #                                     brightness_range=[0.2, 1.2])  #
    #     gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
    #     num_aug = 1
    #
    #
    #     aug_img_stack = []
    #     # labels_stack = []
    #     for image, label in tzip(zip(images, labels)):
    #         aug_img, label_copy = image_tools.augment_helper(datagen, num_aug, 0, image, label)
    #         aug_img = gaussian_noise.augment_images(aug_img) # optional
    #         aug_img_stack.append(aug_img)
    #         # labels_stack.append(label_copy)
    #     name = '/Users/phil/Dropbox/Colab data/H5_data/3lag_80_border_numpy/images_AUG_'+ str(aug_num) +'/' + end_name
    #
    #     np.squeeze(np.asarray(aug_img_stack))
    #     foo_save(name, extracted_images)




################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################

new_keys = 'src_file'  # ull
# src_file - full directory of all source files


ID_keys = ['file_name_nums',
           'frame_nums',
           'in_range',
           'labels',
           'trial_nums_and_frame_nums',
           'full_file_names',]
for key in ID_keys:
    value = image_tools.get_h5_key_and_concatenate(h5_meta_data, key)

    print(key)
    utils.info(value)
    print('________')
    # for k in trial_ind_1:
################################################################################################
################################################################################################
################################################################################################
################################################################################################
################################################################################################
# for k in utils.get_files(bd, '*.npy'):
#     os.rename(k, k.replace('.h5', ''))


bd = '/Users/phil/Dropbox/Colab data/H5_data/3lag_80_border_numpy/'
fold_list = sorted(next(os.walk(bd))[1])

borderINDS_frame_NUMS_images_labels = []
for k in fold_list:
    sorted_files = natsorted(utils.get_files(bd+k, '*.npy'), alg=ns.REAL)
    borderINDS_frame_NUMS_images_labels.append(sorted_files)

for border_inds, frame_nums, images, labels in tqdm(np.asarray(borderINDS_frame_NUMS_images_labels).T):
    # for k in [border_inds, frame_nums, images, labels]:
    #     print(os.path.basename(k))
    # print('______')
    border_inds, frame_nums, images, labels = str(border_inds), str(frame_nums), str(images), str(labels)
    border_inds, frame_nums, images, labels = np.load(border_inds, allow_pickle=True), np.load(frame_nums, allow_pickle=True), np.load(images, allow_pickle=True), np.load(labels, allow_pickle=True)


datagen = ImageDataGenerator(rotation_range=360,  #
                                width_shift_range=.1,  #
                                height_shift_range=.1,  #
                                shear_range=.00,  #
                                zoom_range=.25,
                                brightness_range=[0.2, 1.2])  #
gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
num_aug = 1

ind = 80
aug_img_stack = []
labels_stack = []
for image, label in tqdm(zip(images, labels)):
    aug_img, label_copy = image_tools.augment_helper(datagen, num_aug, 0, image, label)
    aug_img = gaussian_noise.augment_images(aug_img) # optional
    aug_img_stack.append(aug_img)
    labels_stack.append(label_copy)





h5creator.add_to_h5(aug_img_stack[:, :, :, :], labels_stack)
utils.copy_h5_key_to_another_h5(each_h5, new_H5_file, 'frame_nums', 'frame_nums') # copy the frame nums to the sug files
# combine all the
image_tools.split_h5_loop_segments(combine_list,
                                     [1],
                                     each_h5.split('.h5')[0]+'_AUG.h5',
                                     add_numbers_to_name = False,
                                     set_seed=0,
                                     color_channel=True)


"""
#### add frame nums 
"""

bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/80_border/T_V_TS_set_inds'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames/3_border/T_V_TS_set_inds'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_CNN_compare/80_border/T_V_TS_set_inds_CNN_COMPARE/'
replace_string = 'T_V_TS_set_inds'
for f in tqdm(utils.sort(utils.get_files(bd, '*.npy'))):
    data_list = np.load(f, allow_pickle=True)
    frame_nums_list = []
    for k in data_list:
        result, result_ind = utils.group_consecutives(k)
        fn = [len(kk) for kk in result]
        frame_nums_list.append(fn)
    f2 = f.replace(replace_string, 'frame_nums')
    utils.make_path(os.path.dirname(f2))
    np.save(f2, frame_nums_list)



"""
everything about indexing can be done later
just get the images for now 
that mean just get the images for 3 and 80 
upload and start running the 80 through colab
then augment the 3 
then run the 3 augmented 
worry about the labels later tonight  
"""



"""
loop through group inds 
assert they are between some value of the loop segments

"""




"""
##########################################################################################
##########################################################################################
MAKE THE REGULAR IMAGES FROM THE 3LAG IMAGES FOR THE CNN DIRECT COMPARISONS
##########################################################################################
##########################################################################################
"""

bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/images'
replace_string = 'regular_numpy_for_final_CNN_comparison_to_LGBM'
for k in tqdm(utils.get_files(bd, '*.npy')):
    images = np.load(k, allow_pickle=True)
    reg_images = images[:, :, :, 2]
    reg_images = np.repeat(reg_images[:, :, :, None], 3, axis=3)
    foo_save(k.replace('3lag_numpy_aug_for_final_LGBM', replace_string), reg_images)





########



















"""
################################################################################################
################################################################################################
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
SPLIT BY TRIAL 
################################################################################################
################################################################################################
################################################################################################
"""

tmp_label_multiply_by_all = []
split_ratio = [7, 2, 1]
from scipy import stats
ind_naming = '/keep_inds_no_skip_frames/'
ind_naming = 'keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/frame_nums/'
bd2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
all_fn = []
sorted_files = natsorted(utils.get_files(bd, '*.npy'), alg=ns.REAL)
split_trail_frame_nums = []
for k in sorted_files:
    fn = np.load(k, allow_pickle=True)
    all_fn.append(fn)
    mode = stats.mode(fn)[0][0]
    good_vids = fn==mode # drop videos with missing frames

    label_multiply_by_all = []
    np.random.seed(1)
    fn_inds = utils.split_list(range(len(fn)), split_ratio)
    split_trail_frame_nums.append([fn[k] for k in fn_inds])
    for fn_ind in fn_inds:
        label_multiply_by = []
        for ii, num_frames in enumerate(fn):
            if ii in fn_ind and good_vids[ii]:
                label_multiply_by.append([1] * num_frames)
            else:
                label_multiply_by.append([0] * num_frames)
        label_multiply_by = np.concatenate(label_multiply_by)
        label_multiply_by_all.append(label_multiply_by)
    save_name = bd2 +ind_naming+ '/' + os.path.basename(k)
    if 'AH1120_200322' in os.path.basename(k): # remove samsons bad data

        print('AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    ')
        tmp1 = []
        for kk in label_multiply_by_all:
            tmp1.append(kk*0)
        tmp1 = []

        for kk in split_trail_frame_nums[-1]:
            tmp1.append([])
        split_trail_frame_nums[-1] = tmp1
    # tmp_label_multiply_by_all.append(label_multiply_by_all)
    foo_save(save_name, label_multiply_by_all)
utils.save_obj(split_trail_frame_nums,  bd2 +ind_naming+ '/split_trail_frame_nums')





tmp_label_multiply_by_all = np.asarray(tmp_label_multiply_by_all)
tmp_label_multiply_by_all[1][2].shape

tmp1 = tmp_label_multiply_by_all.sum(axis=1)

for ii, k in enumerate(tmp1):
    plt.plot(k+ii)




#
#
# split_ratio = [7, 2, 1]
# from scipy import stats
# ind_naming = '/keep_inds_no_skip_frames/'
# ind_naming = 'INDS'
# bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/frame_nums'
# bd2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
# all_fn = []
# sorted_files = natsorted(utils.get_files(bd, '*.npy'), alg=ns.REAL)
# for k in sorted_files:
#     save_name = bd2 +ind_naming+ '/' + os.path.basename(k)
#     if os.path.isfile(save_name):
#         read_write = 'r+'
#     else:
#         read_write = 'w'
#     with h5py.File(save_name, read_write) as h:
#         fn = np.load(k, allow_pickle=True)
#         h['frame_nums'] = fn ####
#         all_fn.append(fn)
#         mode = stats.mode(fn)[0][0]
#         good_vids = fn==mode
#         h['good_vids'] = good_vids
#         """########### GOOD VIDEOS ##########"""
#         label_multiply_by = []
#             for ii, num_frames in enumerate(fn):
#                 if ii in fn_ind and good_vids[ii]:
#                     label_multiply_by.append([1] * num_frames)
#                 else:
#                     label_multiply_by.append([0] * num_frames)
#             label_multiply_by = np.concatenate(label_multiply_by)
#         if 'AH1120_200322' in os.path.basename(k): # remove samsons bad data
#             print('AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    AH1120_200322    ')
#             label_multiply_by = label_multiply_by_all*0
#         h[''] = label_multiply_by
#         """########### SPLIT BY TRIAL ##########"""
#
#
#
#         """########### 80 BORDER  ##########"""
#
#


split_trail_frame_nums = utils.load_obj('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/split_trail_frame_nums')

def foo_rename(instr):
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]


def foo_border_extraction(y, border):
    b = utils.inds_around_inds(y, border * 2 + 1)
    group_inds, result_ind = utils.group_consecutives(b)
    return group_inds, [len(k) for k in group_inds]

all_h5s = utils.get_h5s(foo_rename('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/'),
                        print_h5_list=False)
h_cont, h_names = utils._get_human_contacts_(all_h5s)
labels = []
for k in h_cont:
    labels.append(1*(np.nanmean(k, axis=0)>.5))

d_list = ['/Users/phil/Dropbox/Colab data/H5_data/regular/',
          '/Users/phil/Dropbox/Colab data/H5_data/3lag/',
          '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS_FINAL_PRED/',
          '/Users/phil/Dropbox/Colab data/H5_data/ALT_LABELS']

h5_meta_images_labels = []
for k in d_list: # just save all the dat to numpy in different folders
    sorted_files = natsorted(utils.get_h5s(k), alg=ns.REAL)
    h5_meta_images_labels.append(sorted_files)



ind_naming = 'keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL'
ind_naming2 = 'keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL_80_border'

bd2 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
save_name = bd2 +ind_naming+ '/'
files = utils.sort(utils.get_files(save_name, '*'))
border = 80
# with h5py.File(bd2 +ind_naming2+'.h5', 'w') as h:
h = dict()
for iii, (f, h5_meta_data, y) in enumerate(zip(files, h5_meta_images_labels[0], labels)):
    bn = os.path.basename(f)

    OG_frame_nums = image_tools.get_h5_key_and_concatenate(h5_meta_data, 'frame_nums')

    group_inds, new_frame_nums = foo_border_extraction(y, border)
    TVt_good_frames = np.load(f, allow_pickle=1)


    border_inds_80 = np.zeros_like(y)
    border_inds_80[np.concatenate(group_inds).astype(int)] = 1

    TVt_good_frames_80_border = [tvt*border_inds_80 for tvt in TVt_good_frames]

    new_fn = []
    for k in TVt_good_frames_80_border:
        k = np.where(k==1)[0]
        group_inds, result_ind = utils.group_consecutives(k)
        new_fn.append([len(kk) for kk in group_inds])

    h[bn] = dict()
    # h[bn]['OG_frame_nums'] = OG_frame_nums

    h[bn]['new_frame_nums'] = new_fn
    h[bn]['TVt_good_frames_80_border'] = TVt_good_frames_80_border
    h[bn]['full_test_set_inds'] = TVt_good_frames[-1]
    h[bn]['full_test_set_frame_nums'] = split_trail_frame_nums[iii][-1]

final_save_name = bd2 + ind_naming2
utils.save_obj(h, final_save_name)



final_save_name = bd2 +ind_naming2
d = utils.load_obj(final_save_name)



split_trail_frame_nums = utils.load_obj('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/split_trail_frame_nums')

replace_data = r'final_2105'
replace_labels = r'labels'
replace_frame_nums = r'frame_nums'

bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
bd_data = bd + replace_data

data_files = utils.sort(utils.get_files(bd_data, '*.npy'))

tvt_x = [[],[],[]]
tvt_y = [[],[],[]]
tvt_fn = [[],[],[]]

'/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/final_2105'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/'
for f_data in tqdm(data_files):
    bn = os.path.basename(f_data)
    d2 = d[bn]

    f_label = f_data.replace(replace_data, replace_labels)
    f_fn = f_data.replace(replace_data, replace_frame_nums)


    f_data = np.load(f_data, allow_pickle=True)
    f_label = np.load(f_label, allow_pickle=True)
    f_fn = np.load(f_fn, allow_pickle=True)


    for set_i in range(3):
        inds = d2['TVt_good_frames_80_border'][set_i]==1
        tvt_x[set_i].append(f_data[inds, :])
        tvt_y[set_i].append(f_label[inds])
        tvt_fn[set_i].append(d2['new_frame_nums'][set_i])
    del f_data

bd3 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/'
for i, k in enumerate(tvt_x):
    tvt_x[i] = np.vstack(k)
for i, k in enumerate(tvt_y):
    tvt_y[i] = np.concatenate(k).astype(int)
for i, k in enumerate(tvt_fn):
    tvt_fn[i] = np.concatenate(k).astype(int)

utils.save_obj(tvt_x, bd3+'/tvt_x')
utils.save_obj(tvt_y, bd3+'/tvt_y')
utils.save_obj(tvt_fn, bd3+'/tvt_fn')



"""
MAKE THE FULL TEST DATA (NOT 80 BORDER) 
SPLIT BY TRIAL 
"""

split_trail_frame_nums = utils.load_obj('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/split_trail_frame_nums')

replace_data = r'final_2105'
replace_labels = r'labels'
replace_frame_nums = r'frame_nums'

bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
bd_data = bd + replace_data

data_files = utils.sort(utils.get_files(bd_data, '*.npy'))
'/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/final_2105'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/'
full_test_data = []
full_test_fn = []
full_test_labels = []
full_test_inds = []
full_test_files_names = []
for f_data in tqdm(data_files):
    bn = os.path.basename(f_data)
    d2 = d[bn]
    # print(d2['full_test_set_inds']))

    f_label = f_data.replace(replace_data, replace_labels)
    f_fn = f_data.replace(replace_data, replace_frame_nums)

    f_data = np.load(f_data, allow_pickle=True)
    f_label = np.load(f_label, allow_pickle=True)
    # f_fn = np.load(f_fn, allow_pickle=True)

    full_test_set_frame_nums = d2['full_test_set_frame_nums']
    inds = d2['full_test_set_inds'] == 1

    full_test_data.append(f_data[inds, :])
    full_test_labels.append(f_label[inds])
    full_test_fn.append(full_test_set_frame_nums)

    full_test_inds.append(inds)
    full_test_files_names.append(bn)
    del f_data
"""last minute add in ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ """
each_file_len = []
for f_data in tqdm(data_files):
    bn = os.path.basename(f_data)
    d2 = d[bn]
    each_file_len.append(np.sum(d2['full_test_set_inds']))
"""last minute add in ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑ """


bd3 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/'
"""##### save the files as pickles #####"""
utils.save_obj(full_test_data, bd3+'/full_test_data')
utils.save_obj(full_test_labels, bd3+'/full_test_labels')
utils.save_obj(full_test_fn, bd3+'/full_test_fn')


full_test_data = np.vstack(full_test_data).astype(np.float64)
full_test_labels = np.concatenate(full_test_labels).astype(np.float32)
full_test_fn = np.concatenate(full_test_fn).astype(np.int32)
full_test_inds = [np.asarray(k).astype(np.int32) for k in full_test_inds]
h5_file_name = bd3+'FINAL_TEST_SPLIT_BY_TRIAL_FULL_TRIALS' + '.h5'
with h5py.File(h5_file_name, 'w') as h:
    # h['full_test_inds'] = full_test_inds
    h['full_test_data'] = full_test_data
    h['full_test_labels'] = full_test_labels
    h['full_test_fn'] = full_test_fn
    for i, k in enumerate(full_test_files_names):
        h[k] = full_test_inds[i]

"""
 ADD THE IMAGE FILES TO MATCH THE FINAL TEST SET
"""
image_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM//images/'
image_files = natsorted(utils.get_files(image_dir, '*.npy'), alg=ns.REAL)
all_images = []
for f, inds in zip(image_files, full_test_inds):
    print(np.sum(inds))
    x = np.load(f, allow_pickle=1)
    all_images.append(x[inds.astype(bool)])
del x
all_images = np.vstack(all_images)

utils.overwrite_h5_key(h5_file_name, 'images', all_images)


no_hair_inds

utils.overwrite_h5_key(h5_file_name, 'each_file_len', each_file_len)

file_frame_counts = np.concatenate([np.asarray([0]), np.cumsum(each_file_len)])
no_hair_inds = np.ones(np.sum(each_file_len)).astype(bool)
no_hair_inds[file_frame_counts[4]:file_frame_counts[5]] = False

utils.overwrite_h5_key(h5_file_name, 'no_hair_inds', no_hair_inds)




"""
now I can tak the video frames and add them and scan through them for pole down mistakes 
"""
# from whacc.touch_curation_GUI import touch_gui
# touch_gui(h5_file_name, 'full_test_labels', label_write_key='XXXXXXXXXXXXXXXXXXXXXXXXpole_in_rangeXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX')


pole_in_range = image_tools.get_h5_key_and_concatenate('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/FINAL_TEST_SPLIT_BY_TRIAL_FULL_TRIALS.h5', 'pole_in_range')
full_test_fn = image_tools.get_h5_key_and_concatenate('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/FINAL_TEST_SPLIT_BY_TRIAL_FULL_TRIALS.h5', 'full_test_fn')
fn_192000 = image_tools.get_h5_key_and_concatenate(h5_file_name, 'fn_192000')

in_range_by_hand = []
in_range_by_hand_frame_nums = []
for ii, (i1, i2) in enumerate(utils.loop_segments(fn_192000)):
    i3 = (i2-i1)/2
    in_range_labels = pole_in_range[i1:i2]
    oor_inds = np.where(in_range_labels==-1)[0]

    pole_in_range_start = np.max(oor_inds[oor_inds<i3])+1
    pole_in_range_end = np.min(oor_inds[oor_inds>i3])-1
    in_range = np.zeros_like(in_range_labels)
    in_range[pole_in_range_start:pole_in_range_end] = 1
    in_range_by_hand.append(in_range)
    in_range_by_hand_frame_nums.append(int(np.sum(in_range)))

in_range_by_hand = np.concatenate(in_range_by_hand)
# utils.overwrite_h5_key(h5_file_name, 'in_range_by_hand', None)
utils.overwrite_h5_key(h5_file_name, 'in_range_by_hand_192000', in_range_by_hand)
utils.overwrite_h5_key(h5_file_name, 'in_range_by_hand_frame_nums', in_range_by_hand_frame_nums)

mod_path = '/Users/phil/Desktop/by_trial_temp_model.pkl'
mod_path = '/Users/phil/Desktop/by_trial_temp_model_with_nans.pkl'
mod = utils.load_obj(mod_path)
full_test_fn = image_tools.get_h5_key_and_concatenate(h5_file_name, 'full_test_fn')
each_file_len = image_tools.get_h5_key_and_concatenate(h5_file_name, 'each_file_len')



fn_192000 = np.concatenate([full_test_fn[:-12], [3000]*6, full_test_fn[-12:]])
utils.overwrite_h5_key(h5_file_name, 'fn_192000', fn_192000)


no_hair_inds = image_tools.get_h5_key_and_concatenate(h5_file_name, 'no_hair_inds')
full_test_fn = image_tools.get_h5_key_and_concatenate(h5_file_name, 'full_test_fn')
fd = image_tools.get_h5_key_and_concatenate(h5_file_name, 'full_test_data')
in_range_by_hand_FULL_192000 = image_tools.get_h5_key_and_concatenate(h5_file_name, 'in_range_by_hand_192000')
labels = image_tools.get_h5_key_and_concatenate(h5_file_name, 'full_test_labels')

# in_range_by_hand_FULL_192000 = np.zeros_like(no_hair_inds)
# in_range_by_hand_FULL_192000[no_hair_inds] = in_range_by_hand

"""
get unique nan feature data 
"""
tmp1 = [np.isnan(k) for k in copy.deepcopy(fd)]
tmp1 = np.asarray(tmp1)
tmp2 = np.unique(tmp1, axis=0)
tmp2.shape

plt.figure(figsize = (20,2))
plt.imshow(tmp2, interpolation='nearest', aspect='auto')

tmp3 = tmp1[:50,:]
plt.figure(figsize = (20,2))
plt.imshow(tmp3, interpolation='nearest', aspect='auto')

start_end_nan_locations = tmp1[4000-50:4000+50,:]
plt.figure(figsize = (20,2))
plt.imshow(start_end_nan_locations, interpolation='nearest', aspect='auto')

utils.save_obj(start_end_nan_locations, '/Users/phil/Dropbox/HIRES_LAB/GitHub/whacc/whacc/whacc_data/feature_data/'+'start_end_nan_locations'+'.pkl')

# """
# remove the noise images
# """
# corrected_images = []
# for ii, (i1, i2) in enumerate(utils.loop_segments(fn_192000)):
#     with h5py.File(h5_file_name, 'r') as h:
#         x = h['images'][i1:i2]
#         asdf
#
#         corrected_images.append(x)

"""
########################
"""

fd = fd[no_hair_inds]
yhat = mod.predict(fd)
utils.overwrite_h5_key(h5_file_name, 'TEMP_yhat', yhat)
x = 1*(yhat>.5)
x = x*in_range_by_hand_FULL_192000

segments, _ = utils.group_consecutives(np.where(x!=labels)[0])
# utils.overwrite_h5_key(h5_file_name, 'TEMP_yhat_full_192000', x)
utils.overwrite_h5_key(h5_file_name, 'TEMP_yhat_full_192000_nan', x)

"""
########################
"""
from scipy.signal import medfilt
yhat = mod.predict(fd)
thresh = .5
yhat_med_5 = []
for i1, i2 in utils.loop_segments(fn_192000):
    x = medfilt(yhat[i1:i2], 5)
    x = 1*(x>thresh)
    yhat_med_5.append(x)
yhat_med_5 = np.concatenate(yhat_med_5)
tmp1 = copy.deepcopy(yhat_med_5)
yhat_med_5 = yhat_med_5*in_range_by_hand_FULL_192000
yhat_med_5 = yhat_med_5*no_hair_inds

utils.overwrite_h5_key(h5_file_name, 'TEMP_yhat_full_192000_nan', yhat_med_5)

plt.plot(tmp1+.2)
plt.plot(TEMP_yhat_full_192000+.1)
plt.plot(labels*no_hair_inds)


"""
########################
"""


plt.plot(in_range_by_hand_FULL_192000)
plt.plot(labels)


# fd2 = []
# for ii, (i1, i2) in enumerate(utils.loop_segments(fn_192000)):
#     x = fd[i1:i2]
#     asdf
#     fd2.append(x)


f2 = np.vstack(f2)


for k in np.asarray(segments)[np.where([len(k)>2 for k in segments])[0]]:
    print(k[0])

from scipy.signal import medfilt
thresh = .5
yhat_med_5 = []
for i1, i2 in utils.loop_segments(full_test_fn):
    x = medfilt(yhat[i1:i2], 5)
    x = 1*(x>thresh)
    yhat_med_5.append(x)
yhat_med_5 = np.concatenate(yhat_med_5)


utils.overwrite_h5_key(h5_file_name, 'TEMP_yhat_med_5', yhat_med_5)
"""
each_file_len
"""


x = yhat_med_5[in_range_by_hand.astype(bool)]
x2 = labels[no_hair_inds][in_range_by_hand.astype(bool)]
plt.plot(x+.2)
plt.plot(x2)
np.sum(no_hair_inds)






# :96000 and 114000:
x = yhat_med_5*in_range_by_hand
plt.plot(x[10*-4000:])




def h5_string_switcher(list_in):
    list_in = utils.make_list(list_in)
    if type(list_in[0]) == bytes:
        print('DECODE switching from bytes to string')
        out = [k.decode("ascii", "ignore") for k in list_in]
    elif type(list_in[0]) == str:
        print('ENCODE switching from string to bytes')
        out = [k.encode("ascii", "ignore") for k in list_in]
    else:
        print('not bytes or string format, returning input')
        return list_in
    return out
utils.h5_string_switcher = h5_string_switcher





for i, A in enumerate(TVt_good_frames_80_border):
    plt.plot(i*.1 + A)
plt.plot(y-1)
for i1, i2 in utils.loop_segments(OG_frame_nums):
    plt.vlines(i1, -1, 1, 'k')


b = utils.inds_around_inds(y, border * 2 + 1)
group_inds, result_ind = utils.group_consecutives(b)

group_inds, [len(k) for k in group_inds]

# b = utils.inds_around_inds(y, border * 2 + 1)
# group_inds, result_ind = utils.group_consecutives(b)
# new_frame_nums = []
# for tmp2 in group_inds:
#     new_frame_nums.append(len(tmp2))

# OG_frame_nums_cumulative = np.cumsum(OG_frame_nums)





from .subset_h5_generator import subset_h5_generator
from .utils import *
from .pole_tracker import PoleTracking
from .touch_curation_GUI import touch_gui
from .analysis import *
from .transfer_learning import *


# from .feature_maker import *
# from .image_tools import *
# from .model_maker import *
# from .touch_curation_GUI import *
#



import os
from whacc import utils
import optuna
import lightgbm as lgb
from sklearn import metrics
from whacc import analysis
import numpy as np
import shutil




class retrain_LGBM():
    def __init__(self, model_base_dir, study_name, tvt_x, tvt_y, tvt_fn, tvt_w, load_optuna_if_exists=False):
        global GLOBALS
        GLOBALS = dict()
        # GLOBALS = GLOBALSin

        # bd = '/Users/phil/Desktop/tmp_mod_test/'
        # study_name = 'my_custom_optuna_models_test_V1'

        GLOBALS['mod_dir'] = model_base_dir + os.sep + study_name + os.sep

        GLOBALS['num_optuna_trials'] = 20  ########  20  3
        GLOBALS['early_stopping_rounds'] = 100  ########  100 10
        GLOBALS['num_iterations'] = 10000 ########  10000 5
        ######################## USER SETTING AFFECTING THE FINAL EVAL RESULTS
        GLOBALS['edge_threshold'] = 5
        GLOBALS['thresholds'] = np.linspace(0.4, .8, 5)
        GLOBALS['smooth_by'] = 5
        ########################
        GLOBALS['tvt_x'] = tvt_x
        GLOBALS['tvt_y'] = tvt_y
        GLOBALS['tvt_fn'] = tvt_fn

        GLOBALS['load_optuna_if_exists'] = load_optuna_if_exists

        GLOBALS['tvt_w'] = tvt_w
        ########################
        GLOBALS['study_name'] = study_name
        GLOBALS['storage_dir'] = GLOBALS['mod_dir']
        GLOBALS['basic_info'] = dict()
        print("""you can change parameters using the name of the class variable plus '.GLOBALS['num_optuna_trials']' for example. see list below for class info""")
        utils.info(GLOBALS)
        self.GLOBALS = GLOBALS

    def set_global(self):
        GLOBALS = self.GLOBALS

    def train_model(self):
        self.set_global()

        mod_dir = GLOBALS['mod_dir']
        if os.path.isdir(mod_dir):
            if GLOBALS['load_optuna_if_exists']:
                print('Attempting to load previous optuna training file...')
            else:
                inp = input('directory exists already, type "delete" to delete existing training folder, cancel or type anything else to stop training')
                if inp.lower() == 'delete':
                    shutil.rmtree(mod_dir)
                    print('directory was deleted and remade ready to run!')
                else:
                    assert False, "directory already exists stopping training "

        utils.make_path(mod_dir)

    # if __name__ == "__main__":
        study_name = GLOBALS['study_name']
        storage_dir = GLOBALS['storage_dir']

        storage = f"sqlite:///{storage_dir}/OPTUNA_SAVE_{study_name}.db"

        study = optuna.create_study(study_name=study_name, storage=storage, direction="maximize",
                                    load_if_exists=GLOBALS['load_optuna_if_exists'])

        study.optimize(self.objective, n_trials=GLOBALS['num_optuna_trials'], timeout=None, n_jobs=1,
                       show_progress_bar=True)

        print("Number of finished trials: {}".format(len(study.trials)))

        print("Best trial:")
        trial = study.best_trial

        print("  Value: {}".format(trial.value))

        print("  Params: ")
        for key, value in trial.params.items():
            print("    {}: {}".format(key, value))


    @staticmethod
    def objective(trial):
        def reset_metrics():
            def callback(env):
                mod_dir = GLOBALS['mod_dir']
                edge_threshold = GLOBALS['edge_threshold']
                thresholds = GLOBALS['thresholds']
                smooth_by = GLOBALS['smooth_by']

                real = GLOBALS['tvt_y'][2]
                touch_count = np.sum(np.diff(real) == 1)
                frame_nums = GLOBALS['tvt_fn'][2]
                yhat = env.model.predict(GLOBALS['tvt_x'][2])
                yhat_proba = utils.smooth(yhat, smooth_by)
                df = analysis.thresholded_error_types(real, yhat_proba, edge_threshold=edge_threshold,
                                                      frame_num_array=frame_nums, thresholds=thresholds)
                df2 = df.iloc[:, :-2] / touch_count
                x = df2.sum(axis=1)
                err = x.min()

                GLOBALS['all_errors_test'].append(df)
                GLOBALS['metrics_test']['auc'].append(metrics.roc_auc_score(real, yhat))
                GLOBALS['metrics_test']['touch_count_errors_per_touch'].append(err)
                GLOBALS['metrics_test']['best_threshold'].append(thresholds[x.argmin()])

                # do the sam for the trianing data
                real = GLOBALS['tvt_y'][0]
                touch_count = np.sum(np.diff(real) == 1)
                frame_nums = GLOBALS['tvt_fn'][0]
                yhat = env.model.predict(GLOBALS['tvt_x'][0])
                yhat_proba = utils.smooth(yhat, smooth_by)
                df = analysis.thresholded_error_types(real, yhat_proba, edge_threshold=edge_threshold,
                                                      frame_num_array=frame_nums, thresholds=thresholds)
                df2 = df.iloc[:, :-2] / touch_count
                x = df2.sum(axis=1)
                err = x.min()

                GLOBALS['all_errors_train'].append(df)
                GLOBALS['metrics_train']['auc'].append(metrics.roc_auc_score(real, yhat))
                GLOBALS['metrics_train']['touch_count_errors_per_touch'].append(err)
                GLOBALS['metrics_train']['best_threshold'].append(thresholds[x.argmin()])

            return callback
        def thresholded_error_types(yhat, dmat):
            ######################## SHOULDNT CHANGE
            frame_nums_val = GLOBALS['tvt_fn'][1]
            mod_dir = GLOBALS['mod_dir']
            higher_is_better = False
            edge_threshold = GLOBALS['edge_threshold']
            thresholds = GLOBALS['thresholds']
            smooth_by = GLOBALS['smooth_by']
            ######################## SHOULDNT CHANGE -- CALC TCerr
            yhat_proba = utils.smooth(yhat, smooth_by)
            real = dmat.get_label() if isinstance(dmat, lgb.Dataset) else dmat
            touch_count = np.sum(np.diff(real) == 1)
            #     with HiddenPrints():
            df = analysis.thresholded_error_types(real, yhat_proba,
                                                  edge_threshold=edge_threshold,
                                                  frame_num_array=frame_nums_val,
                                                  thresholds=thresholds)
            df2 = df.iloc[:, :-2] / touch_count
            x = df2.sum(axis=1)
            err = x.min()
            ######################## THE REST IS SAVING AND RETURN FOR LGBM EVAL
            GLOBALS['all_errors'].append(df)
            GLOBALS['metrics_val']['auc'].append(metrics.roc_auc_score(real, yhat))
            GLOBALS['metrics_val']['touch_count_errors_per_touch'].append(err)
            GLOBALS['metrics_val']['best_threshold'].append(thresholds[x.argmin()])
            if 'basic_info' not in list(GLOBALS.keys()):
                GLOBALS['basic_info'] = dict()
            GLOBALS['basic_info']['threshold'] = thresholds
            GLOBALS['basic_info']['higher_is_better'] = higher_is_better
            GLOBALS['basic_info']['edge_threshold'] = edge_threshold
            GLOBALS['basic_info']['touch_count'] = touch_count
            GLOBALS['basic_info']['real_human_predictions'] = real
            GLOBALS['basic_info']['smooth_by'] = smooth_by
            GLOBALS['basic_info']['num_optuna_trials'] = GLOBALS['num_optuna_trials']
            GLOBALS['basic_info']['early_stopping_rounds'] = GLOBALS['early_stopping_rounds']
            GLOBALS['basic_info']['num_iterations'] = GLOBALS['num_iterations']
            GLOBALS['basic_info']['num_optuna_trials'] = GLOBALS['num_optuna_trials']
            return "touch_count_errors", err, higher_is_better

        d = utils.load_feature_data()
        names = np.asarray(d['full_feature_names_and_neuron_nums'])
        final_feature_names_USE = list(names[d['final_selected_features_bool']])
        if len(final_feature_names_USE) != GLOBALS['tvt_x'][0].shape[-1]:
            print('not using the 2105 features dataset, removing feature names and replacing with numbers')
            final_feature_names_USE = [str(k).zfill(4) for k in np.arange(GLOBALS['tvt_x'][0].shape[-1])]

        train_DATA = lgb.Dataset(GLOBALS['tvt_x'][0], label=GLOBALS['tvt_y'][0],
                                 feature_name=final_feature_names_USE, weight=GLOBALS['tvt_w'][0])
        val_DATA = lgb.Dataset(GLOBALS['tvt_x'][1], label=GLOBALS['tvt_y'][1],
                               feature_name=final_feature_names_USE, reference=train_DATA,
                               weight=GLOBALS['tvt_w'][1])

        param = {
            "objective": "binary",
            "metric": "auc",
            "verbosity": -1,
            "boosting_type": "gbdt",
            "early_stopping_rounds": GLOBALS['early_stopping_rounds'],
            # "num_boost_round": GLOBALS['num_iterations'],
            "lambda_l1": trial.suggest_float("lambda_l1", 1e-8, 10.0, log=True),
            "lambda_l2": trial.suggest_float("lambda_l2", 1e-8, 10.0, log=True),
            "num_leaves": trial.suggest_int("num_leaves", 2, 256),
            "feature_fraction": trial.suggest_float("feature_fraction", 0.4, 1.0),
            "bagging_fraction": trial.suggest_float("bagging_fraction", 0.4, 1.0),
            "bagging_freq": trial.suggest_int("bagging_freq", 1, 7),
            "min_child_samples": trial.suggest_int("min_child_samples", 5, 100),
            #         'first_metric_only' : True,
        }

        ######################## INIT THE LISTS FOR SAVING LATER

        GLOBALS['metrics_val'] = dict()
        GLOBALS['metrics_val']['auc'] = []
        GLOBALS['metrics_val']['touch_count_errors_per_touch'] = []
        GLOBALS['metrics_val']['best_threshold'] = []
        GLOBALS['all_errors'] = []

        GLOBALS['metrics_test'] = dict()
        GLOBALS['metrics_test']['auc'] = []
        GLOBALS['metrics_test']['touch_count_errors_per_touch'] = []
        GLOBALS['metrics_test']['best_threshold'] = []
        GLOBALS['all_errors_test'] = []

        GLOBALS['metrics_train'] = dict()
        GLOBALS['metrics_train']['auc'] = []
        GLOBALS['metrics_train']['touch_count_errors_per_touch'] = []
        GLOBALS['metrics_train']['best_threshold'] = []
        GLOBALS['all_errors_train'] = []

        gbm = lgb.train(
            param,
            train_DATA,
            num_boost_round=GLOBALS['num_iterations'],
            valid_sets=[val_DATA],
            feval=thresholded_error_types,
            callbacks=[reset_metrics()],
        )

        preds = gbm.predict(GLOBALS['tvt_x'][1])
        fpr, tpr, thresholds_auc = metrics.roc_curve(GLOBALS['tvt_y'][1], preds, pos_label=1)
        AUC_out = metrics.auc(fpr, tpr)

        # save the model
        mod_num = str(len(utils.get_files(GLOBALS['mod_dir'], '*_model.pkl'))).zfill(3)
        fn = GLOBALS['mod_dir'] + mod_num + '_model.pkl'
        utils.save_obj(gbm, fn)

        d = dict()
        d['metrics'] = GLOBALS['metrics_val']
        d['all_errors'] = GLOBALS['all_errors']
        d['metrics_test'] = GLOBALS['metrics_test']
        d['all_errors_test'] = GLOBALS['all_errors_test']

        fn = GLOBALS['mod_dir'] + mod_num + '_model_results.pkl'
        utils.save_obj(d, fn)

        if mod_num == '000':
            fn = GLOBALS['mod_dir'] + 'basic_info.pkl'
            utils.save_obj(GLOBALS['basic_info'], fn)

        return AUC_out

